import array
import sys

import Foundation
import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestString(TestCase):
    def testType(self):
        self.assertTrue(issubclass(CoreFoundation.CFStringRef, Foundation.NSString))

    def testTypeID(self):
        v = CoreFoundation.CFStringGetTypeID()
        self.assertIsInstance(v, int)

    def testNoPascalStrings(self):
        self.assertNotHasAttr(CoreFoundation, "CFStringCreateWithPascalString")
        self.assertNotHasAttr(CoreFoundation, "CFStringCreateWithPascalStringNoCopy")
        self.assertNotHasAttr(CoreFoundation, "CFStringGetPascalString")
        self.assertNotHasAttr(CoreFoundation, "CFStringGetPascalStringPtr")
        self.assertNotHasAttr(CoreFoundation, "CFStringAppendPascalString")

    def testCreation(self):
        s = CoreFoundation.CFStringCreateWithCString(
            None, b"hello world", CoreFoundation.kCFStringEncodingASCII
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "hello world")

        s = CoreFoundation.CFStringCreateWithBytes(
            None, b"hello world", 5, CoreFoundation.kCFStringEncodingASCII, False
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "hello")

        s = CoreFoundation.CFStringCreateWithCharacters(None, "HELLO", 5)
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "HELLO")

        # NOTE: the deallocator must be CoreFoundation.kCFAllocatorNull
        s = CoreFoundation.CFStringCreateWithCStringNoCopy(
            None,
            b"hello world",
            CoreFoundation.kCFStringEncodingASCII,
            CoreFoundation.kCFAllocatorNull,
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "hello world")

        s = CoreFoundation.CFStringCreateWithBytesNoCopy(
            None,
            b"hello world",
            5,
            CoreFoundation.kCFStringEncodingASCII,
            False,
            CoreFoundation.kCFAllocatorNull,
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "hello")

        s = CoreFoundation.CFStringCreateWithCharactersNoCopy(
            None, "HELLO", 5, CoreFoundation.kCFAllocatorNull
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertIsInstance(s, str)
        self.assertEqual(s, "HELLO")

        del s

        s = CoreFoundation.CFStringCreateWithSubstring(
            None, "Hello world", CoreFoundation.CFRange(2, 4)
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "Hello world"[2:6])

        s = CoreFoundation.CFStringCreateCopy(None, "foo the bar")
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "foo the bar")

        s = CoreFoundation.CFStringCreateWithFormat(
            None, None, "hello %s = %d", b"foo", 52
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "hello foo = 52")

        self.assertFalse(
            hasattr(CoreFoundation, "CFStringCreateWithFormatAndArguments")
        )

    def testCreateMutable(self):
        s = CoreFoundation.CFStringCreateMutable(None, 0)
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "")

        s = CoreFoundation.CFStringCreateMutableCopy(None, 0, "foobar")
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "foobar")

        if sys.version_info[:2] < (3, 3):
            # See issue15035 on Python's tracker. There currently is
            # no array format code for UCS2 storage (Python 3.3 or later)
            # Therefore these tests can only work for earlier versions
            # of Python.

            b = array.array("u", "hello world")
            s = CoreFoundation.CFStringCreateMutableWithExternalCharactersNoCopy(
                None, b, len(b), len(b), CoreFoundation.kCFAllocatorNull
            )
            self.assertIsInstance(s, objc.pyobjc_unicode)
            self.assertEqual(s, "hello world")

            b[0] = "H"

            # The objc_str proxy is immutable
            self.assertEqual(s, "hello world")

            # The changed string can be accessed directly though:
            s = s.nsstring()
            self.assertEqual(s, "Hello world")

            b2 = array.array("u", " " * 40)
            CoreFoundation.CFStringSetExternalCharactersNoCopy(s, b2, len(s), 40)
            self.assertEqual(s, " " * len(b))
            b2[0] = "X"
            self.assertEqual(s, "X" + (" " * (len(b) - 1)))

    def testFunctions(self):
        v = CoreFoundation.CFStringGetLength("bla bla")
        self.assertEqual(v, 7)

        v = CoreFoundation.CFStringGetCharacterAtIndex("zing", 2)
        self.assertIsInstance(v, str)
        self.assertIsNotInstance(v, objc.pyobjc_unicode)
        self.assertEqual(v, "n")

        v = CoreFoundation.CFStringGetCharacters(
            "foo", CoreFoundation.CFRange(0, 3), None
        )
        self.assertIsInstance(v, str)
        self.assertIsNotInstance(v, objc.pyobjc_unicode)
        self.assertEqual(v, "foo")

        ok, buf = CoreFoundation.CFStringGetCString(
            "sing along", None, 100, CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertTrue(ok)

        # Annoyingly metadata cannot represent the exact interface
        self.assertIsInstance(buf, bytes)
        if b"\0" in buf:
            buf = buf[: buf.index(b"\0")]
        self.assertEqual(buf, b"sing along")
        s = CoreFoundation.CFStringGetCStringPtr(
            "apenootjes", CoreFoundation.kCFStringEncodingASCII
        )
        if s is not objc.NULL:
            self.assertEqual(s, b"apenootjes")
            self.assertIsInstance(s, str)
        s = CoreFoundation.CFStringGetCharactersPtr("apenootjes")
        if s is not objc.NULL:
            self.assertEqual(s, "apenootjes")
            self.assertIsInstance(s, str)
        idx, buf, used = CoreFoundation.CFStringGetBytes(
            "apenootjes",
            CoreFoundation.CFRange(1, 4),
            CoreFoundation.kCFStringEncodingASCII,
            b"\0",
            True,
            None,
            50,
            None,
        )
        self.assertEqual(idx, 4)
        self.assertIsInstance(buf, bytes)
        self.assertEqual(buf, b"peno")
        self.assertEqual(used, 4)

        s = CoreFoundation.CFStringCreateFromExternalRepresentation(
            None, b"hello world", CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "hello world")

        data = CoreFoundation.CFStringCreateExternalRepresentation(
            None, s, CoreFoundation.kCFStringEncodingUTF16BE, b"\0"
        )
        self.assertIsInstance(data, CoreFoundation.CFDataRef)
        val = CoreFoundation.CFDataGetBytes(
            data, (0, CoreFoundation.CFDataGetLength(data)), None
        )
        self.assertEqual(val, b"\0h\0e\0l\0l\0o\0 \0w\0o\0r\0l\0d")

        v = CoreFoundation.CFStringGetSmallestEncoding(s)
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFStringGetFastestEncoding(s)
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFStringGetSystemEncoding()
        self.assertIsInstance(v, int)
        v = CoreFoundation.CFStringGetMaximumSizeForEncoding(
            100, CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertIsInstance(v, int)
        ok, buf = CoreFoundation.CFStringGetFileSystemRepresentation(
            "/path/to/nowhere.txt", None, 100
        )
        self.assertTrue(ok)
        if b"\0" in buf:
            buf = buf[: buf.index(b"\0")]
        self.assertEqual(buf, b"/path/to/nowhere.txt")
        self.assertIsInstance(buf, bytes)
        idx = CoreFoundation.CFStringGetMaximumSizeOfFileSystemRepresentation("/tmp")
        self.assertIsInstance(idx, int)
        s = CoreFoundation.CFStringCreateWithFileSystemRepresentation(None, b"/tmp")
        self.assertIsInstance(s, objc.pyobjc_unicode)
        self.assertEqual(s, "/tmp")
        self.assertRaises(
            (TypeError, ValueError),
            CoreFoundation.CFStringCreateWithFileSystemRepresentation,
            None,
            "/tmp",
        )

        r = CoreFoundation.CFStringCompareWithOptionsAndLocale(
            "aas",
            "noot",
            CoreFoundation.CFRange(0, 3),
            CoreFoundation.kCFCompareBackwards,
            CoreFoundation.CFLocaleCopyCurrent(),
        )
        self.assertEqual(r, CoreFoundation.kCFCompareLessThan)

        r = CoreFoundation.CFStringCompareWithOptions(
            "aap", "noot", CoreFoundation.CFRange(0, 3), 0
        )
        self.assertEqual(r, CoreFoundation.kCFCompareLessThan)

        r = CoreFoundation.CFStringCompare(
            "aap", "AAP", CoreFoundation.kCFCompareCaseInsensitive
        )
        self.assertEqual(r, CoreFoundation.kCFCompareEqualTo)

        found, rng = CoreFoundation.CFStringFindWithOptionsAndLocale(
            "the longer string",
            "longer",
            CoreFoundation.CFRange(0, 17),
            0,
            CoreFoundation.CFLocaleCopyCurrent(),
            None,
        )
        self.assertIs(found, True)
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        self.assertEqual(rng, CoreFoundation.CFRange(4, 6))

        found, rng = CoreFoundation.CFStringFindWithOptions(
            "the longer string", "longer", CoreFoundation.CFRange(0, 17), 0, None
        )
        self.assertIs(found, True)
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        self.assertEqual(rng, CoreFoundation.CFRange(4, 6))

        arr = CoreFoundation.CFStringCreateArrayWithFindResults(
            None, "word a world a world", "world", CoreFoundation.CFRange(0, 20), 0
        )
        self.assertIsInstance(arr, CoreFoundation.CFArrayRef)
        self.assertEqual(len(arr), 2)

        rng = CoreFoundation.CFStringFind("hello world", "world", 0)
        self.assertEqual(rng, CoreFoundation.CFRange(6, 5))

        ok = CoreFoundation.CFStringHasPrefix("hello", "he")
        self.assertIs(ok, True)
        ok = CoreFoundation.CFStringHasPrefix("hello", "ge")
        self.assertIs(ok, False)
        ok = CoreFoundation.CFStringHasSuffix("hello", "lo")
        self.assertIs(ok, True)
        rng = CoreFoundation.CFStringGetRangeOfComposedCharactersAtIndex(
            "hello world", 5
        )
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        found, rng = CoreFoundation.CFStringFindCharacterFromSet(
            "hello  world",
            CoreFoundation.CFCharacterSetGetPredefined(
                CoreFoundation.kCFCharacterSetWhitespace
            ),
            CoreFoundation.CFRange(0, 12),
            0,
            None,
        )
        self.assertIs(found, True)
        self.assertIsInstance(rng, CoreFoundation.CFRange)
        (
            lineBeginIndex,
            lineEndIndex,
            contentsEndIndex,
        ) = CoreFoundation.CFStringGetLineBounds(
            "hello\n\nworld", CoreFoundation.CFRange(0, 12), None, None, None
        )
        self.assertEqual(lineBeginIndex, 0)
        self.assertEqual(lineEndIndex, 12)
        self.assertEqual(contentsEndIndex, 12)

        (
            paraBeginIndex,
            paraEndIndex,
            contentsEndIndex,
        ) = CoreFoundation.CFStringGetParagraphBounds(  # noqa: B950
            "hello\n\nworld", CoreFoundation.CFRange(0, 12), None, None, None
        )
        self.assertEqual(paraBeginIndex, 0)
        self.assertEqual(paraEndIndex, 12)
        self.assertEqual(contentsEndIndex, 12)

        result = CoreFoundation.CFStringCreateByCombiningStrings(
            None, ["hello", "world"], "//"
        )
        self.assertEqual(result, "hello//world")

        result = CoreFoundation.CFStringCreateArrayBySeparatingStrings(
            None, "hello world", " "
        )
        self.assertEqual(result, ["hello", "world"])

        v = CoreFoundation.CFStringGetIntValue("1000")
        self.assertEqual(v, 1000)

        v = CoreFoundation.CFStringGetDoubleValue("1000")
        self.assertEqual(v, 1000.0)

        v = CoreFoundation.CFStringGetDoubleValue("1000.5")
        self.assertEqual(v, 1000.5)

    def testMutableFunctions(self):
        s = CoreFoundation.CFStringCreateMutable(None, 0)
        s = s.nsstring()
        CoreFoundation.CFStringAppend(s, "hello")
        self.assertEqual(s, "hello")

        s = CoreFoundation.CFStringCreateMutable(None, 0)
        # Ensure that we actually see updates:
        s = s.nsstring()

        CoreFoundation.CFStringAppendCharacters(s, "hel", 3)
        self.assertEqual(s, "hel")

        CoreFoundation.CFStringAppendCString(
            s, b"lo world", CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertEqual(s, "hello world")

        CoreFoundation.CFStringAppendFormat(s, None, ": %s = %d", b"x", 99)
        self.assertEqual(s, "hello world: x = 99")

        self.assertNotHasAttr(CoreFoundation, "CFStringAppendFormatAndArguments")
        CoreFoundation.CFStringInsert(s, 2, "--")
        self.assertEqual(s, "he--llo world: x = 99")

        CoreFoundation.CFStringDelete(s, CoreFoundation.CFRange(0, 8))
        self.assertEqual(s, "world: x = 99")

        CoreFoundation.CFStringReplace(s, CoreFoundation.CFRange(0, 4), "WOR")
        self.assertEqual(s, "WORd: x = 99")

        CoreFoundation.CFStringReplaceAll(s, "WHOOPS")
        self.assertEqual(s, "WHOOPS")

        CoreFoundation.CFStringReplaceAll(s, "BLAblaBLAblaBLA")
        self.assertEqual(s, "BLAblaBLAblaBLA")

        CoreFoundation.CFStringFindAndReplace(
            s, "BL", " fo ", CoreFoundation.CFRange(0, 15), 0
        )
        self.assertEqual(s, " fo Abla fo Abla fo A")

        CoreFoundation.CFStringReplaceAll(s, "abc")
        CoreFoundation.CFStringPad(s, " ", 9, 0)
        self.assertEqual(s, "abc      ")

        CoreFoundation.CFStringReplaceAll(s, "abc")
        CoreFoundation.CFStringPad(s, ". ", 9, 1)
        self.assertEqual(s, "abc . . .")

        CoreFoundation.CFStringReplaceAll(s, "abcdef")
        CoreFoundation.CFStringPad(s, ". ", 3, 0)
        self.assertEqual(s, "abc")

        CoreFoundation.CFStringReplaceAll(s, "aHelloaaa")
        trim_chars = "a"

        # print s
        CoreFoundation.CFStringTrim(s, trim_chars)
        # print s
        # print(s)
        # CoreFoundation.CFShow(s)
        self.assertEqual(s, "Hello")

        CoreFoundation.CFStringReplaceAll(s, "* * * *abc * ")
        trim_chars = "* "

        trim_chars = CoreFoundation.CFStringCreateWithCString(
            None, b"* ", CoreFoundation.kCFStringEncodingASCII
        )
        CoreFoundation.CFStringTrim(s, trim_chars)
        self.assertEqual(s, "*abc ")

        CoreFoundation.CFStringReplaceAll(s, " \tHello world  \t ")
        CoreFoundation.CFStringTrimWhitespace(s)
        self.assertEqual(s, "Hello world")

        CoreFoundation.CFStringReplaceAll(s, "AbC")
        CoreFoundation.CFStringLowercase(s, CoreFoundation.CFLocaleCopyCurrent())
        self.assertEqual(s, "abc")

        CoreFoundation.CFStringReplaceAll(s, "AbC")
        CoreFoundation.CFStringUppercase(s, CoreFoundation.CFLocaleCopyCurrent())
        self.assertEqual(s, "ABC")

        CoreFoundation.CFStringReplaceAll(s, "hello world")
        CoreFoundation.CFStringCapitalize(s, CoreFoundation.CFLocaleCopyCurrent())
        self.assertEqual(s, "Hello World")

        CoreFoundation.CFStringNormalize(s, CoreFoundation.kCFStringNormalizationFormKD)
        self.assertEqual(s, "Hello World")

        CoreFoundation.CFStringFold(
            s,
            CoreFoundation.kCFCompareCaseInsensitive,
            CoreFoundation.CFLocaleCopyCurrent(),
        )
        self.assertEqual(s, "hello world")

        CoreFoundation.CFStringReplaceAll(s, "A C")
        ok, rng = CoreFoundation.CFStringTransform(
            s,
            CoreFoundation.CFRange(0, 3),
            CoreFoundation.kCFStringTransformToXMLHex,
            False,
        )
        self.assertEqual(s, "A C")
        self.assertIs(ok, True)
        self.assertEqual(rng, CoreFoundation.CFRange(0, 3))

    def testStringEncoding(self):
        ok = CoreFoundation.CFStringIsEncodingAvailable(
            CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertIs(ok, True)
        encodings = CoreFoundation.CFStringGetListOfAvailableEncodings()
        self.assertIsInstance(encodings, objc.varlist)
        for e in encodings:
            if e == CoreFoundation.kCFStringEncodingInvalidId:
                break
            self.assertIsInstance(e, int)
        s = CoreFoundation.CFStringGetNameOfEncoding(
            CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertEqual(s, "Unicode (UTF-8)")

        v = CoreFoundation.CFStringConvertEncodingToNSStringEncoding(
            CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertIsInstance(v, int)
        t = CoreFoundation.CFStringConvertNSStringEncodingToEncoding(v)
        self.assertEqual(t, CoreFoundation.kCFStringEncodingUTF8)

        v = CoreFoundation.CFStringConvertEncodingToWindowsCodepage(
            CoreFoundation.kCFStringEncodingISOLatin1
        )
        self.assertIsInstance(v, int)
        t = CoreFoundation.CFStringConvertWindowsCodepageToEncoding(v)
        self.assertEqual(t, CoreFoundation.kCFStringEncodingISOLatin1)

        v = CoreFoundation.CFStringConvertEncodingToIANACharSetName(
            CoreFoundation.kCFStringEncodingUTF8
        )
        self.assertIsInstance(v, str)
        self.assertIn(v, ("UTF-8", "utf-8"))

        t = CoreFoundation.CFStringConvertIANACharSetNameToEncoding(v)
        self.assertEqual(t, CoreFoundation.kCFStringEncodingUTF8)

        v = CoreFoundation.CFStringGetMostCompatibleMacStringEncoding(
            CoreFoundation.kCFStringEncodingWindowsLatin1
        )
        self.assertEqual(v, CoreFoundation.kCFStringEncodingMacRoman)

    def testNoInlineBuffer(self):
        self.assertNotHasAttr(CoreFoundation, "CFStringInlineBuffer")
        self.assertNotHasAttr(CoreFoundation, "CFStringInitInlineBuffer")
        self.assertNotHasAttr(CoreFoundation, "CFStringGetCharacterFromInlineBuffer")

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFStringEncodingInvalidId, 0xFFFFFFFF)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacRoman, 0)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsLatin1, 0x0500)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin1, 0x0201)
        self.assertEqual(CoreFoundation.kCFStringEncodingNextStepLatin, 0x0B01)
        self.assertEqual(CoreFoundation.kCFStringEncodingASCII, 0x0600)
        self.assertEqual(CoreFoundation.kCFStringEncodingUnicode, 0x0100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF8, 0x08000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingNonLossyASCII, 0x0BFF)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF16, 0x0100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF16BE, 0x10000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF16LE, 0x14000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF32, 0x0C000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF32BE, 0x18000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF32LE, 0x1C000100)
        self.assertEqual(CoreFoundation.kCFCompareCaseInsensitive, 1)
        self.assertEqual(CoreFoundation.kCFCompareBackwards, 4)
        self.assertEqual(CoreFoundation.kCFCompareAnchored, 8)
        self.assertEqual(CoreFoundation.kCFCompareNonliteral, 16)
        self.assertEqual(CoreFoundation.kCFCompareLocalized, 32)
        self.assertEqual(CoreFoundation.kCFCompareNumerically, 64)
        self.assertEqual(CoreFoundation.kCFCompareDiacriticInsensitive, 128)
        self.assertEqual(CoreFoundation.kCFCompareWidthInsensitive, 256)
        self.assertEqual(CoreFoundation.kCFCompareForcedOrdering, 512)
        self.assertEqual(CoreFoundation.kCFStringNormalizationFormD, 0)
        self.assertEqual(CoreFoundation.kCFStringNormalizationFormKD, 1)
        self.assertEqual(CoreFoundation.kCFStringNormalizationFormC, 2)
        self.assertEqual(CoreFoundation.kCFStringNormalizationFormKC, 3)
        self.assertIsInstance(CoreFoundation.kCFStringTransformStripCombiningMarks, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformToLatin, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformFullwidthHalfwidth, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinKatakana, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinHiragana, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformHiraganaKatakana, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformMandarinLatin, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinHangul, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinArabic, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinHebrew, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinThai, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinCyrillic, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformLatinGreek, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformToXMLHex, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformToUnicodeName, str)
        self.assertIsInstance(CoreFoundation.kCFStringTransformStripDiacritics, str)

    def testNoPrivate(self):
        self.assertNotHasAttr(CoreFoundation, "__CFStringMakeConstantString")

    def testCFSTR(self):
        v = CoreFoundation.CFSTR("hello")
        self.assertIsInstance(v, str)


class TestStringEncodingExt(TestCase):
    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF7, 0x04000100)
        self.assertEqual(CoreFoundation.kCFStringEncodingUTF7_IMAP, 0x0A10)

    def testConstants(self):
        self.assertEqual(CoreFoundation.kCFStringEncodingMacJapanese, 1)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacChineseTrad, 2)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacKorean, 3)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacArabic, 4)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacHebrew, 5)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacGreek, 6)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacCyrillic, 7)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacDevanagari, 9)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacGurmukhi, 10)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacGujarati, 11)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacOriya, 12)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacBengali, 13)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacTamil, 14)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacTelugu, 15)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacKannada, 16)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacMalayalam, 17)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacSinhalese, 18)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacBurmese, 19)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacKhmer, 20)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacThai, 21)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacLaotian, 22)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacGeorgian, 23)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacArmenian, 24)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacChineseSimp, 25)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacTibetan, 26)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacMongolian, 27)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacEthiopic, 28)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacCentralEurRoman, 29)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacVietnamese, 30)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacExtArabic, 31)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacSymbol, 33)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacDingbats, 34)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacTurkish, 35)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacCroatian, 36)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacIcelandic, 37)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacRomanian, 38)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacCeltic, 39)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacGaelic, 40)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacFarsi, 0x8C)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacUkrainian, 0x98)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacInuit, 0xEC)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacVT100, 0xFC)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacHFS, 0xFF)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin2, 0x0202)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin3, 0x0203)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin4, 0x0204)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatinCyrillic, 0x0205)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatinArabic, 0x0206)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatinGreek, 0x0207)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatinHebrew, 0x0208)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin5, 0x0209)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin6, 0x020A)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatinThai, 0x020B)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin7, 0x020D)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin8, 0x020E)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin9, 0x020F)
        self.assertEqual(CoreFoundation.kCFStringEncodingISOLatin10, 0x0210)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSLatinUS, 0x0400)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSGreek, 0x0405)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSBalticRim, 0x0406)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSLatin1, 0x0410)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSGreek1, 0x0411)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSLatin2, 0x0412)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSCyrillic, 0x0413)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSTurkish, 0x0414)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSPortuguese, 0x0415)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSIcelandic, 0x0416)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSHebrew, 0x0417)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSCanadianFrench, 0x0418)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSArabic, 0x0419)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSNordic, 0x041A)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSRussian, 0x041B)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSGreek2, 0x041C)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSThai, 0x041D)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSJapanese, 0x0420)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSChineseSimplif, 0x0421)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSKorean, 0x0422)
        self.assertEqual(CoreFoundation.kCFStringEncodingDOSChineseTrad, 0x0423)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsLatin2, 0x0501)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsCyrillic, 0x0502)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsGreek, 0x0503)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsLatin5, 0x0504)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsHebrew, 0x0505)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsArabic, 0x0506)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsBalticRim, 0x0507)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsVietnamese, 0x0508)
        self.assertEqual(CoreFoundation.kCFStringEncodingWindowsKoreanJohab, 0x0510)
        self.assertEqual(CoreFoundation.kCFStringEncodingANSEL, 0x0601)
        self.assertEqual(CoreFoundation.kCFStringEncodingJIS_X0201_76, 0x0620)
        self.assertEqual(CoreFoundation.kCFStringEncodingJIS_X0208_83, 0x0621)
        self.assertEqual(CoreFoundation.kCFStringEncodingJIS_X0208_90, 0x0622)
        self.assertEqual(CoreFoundation.kCFStringEncodingJIS_X0212_90, 0x0623)
        self.assertEqual(CoreFoundation.kCFStringEncodingJIS_C6226_78, 0x0624)
        self.assertEqual(CoreFoundation.kCFStringEncodingShiftJIS_X0213, 0x0628)
        self.assertEqual(
            CoreFoundation.kCFStringEncodingShiftJIS_X0213_MenKuTen, 0x0629
        )
        self.assertEqual(CoreFoundation.kCFStringEncodingGB_2312_80, 0x0630)
        self.assertEqual(CoreFoundation.kCFStringEncodingGBK_95, 0x0631)
        self.assertEqual(CoreFoundation.kCFStringEncodingGB_18030_2000, 0x0632)
        self.assertEqual(CoreFoundation.kCFStringEncodingKSC_5601_87, 0x0640)
        self.assertEqual(CoreFoundation.kCFStringEncodingKSC_5601_92_Johab, 0x0641)
        self.assertEqual(CoreFoundation.kCFStringEncodingCNS_11643_92_P1, 0x0651)
        self.assertEqual(CoreFoundation.kCFStringEncodingCNS_11643_92_P2, 0x0652)
        self.assertEqual(CoreFoundation.kCFStringEncodingCNS_11643_92_P3, 0x0653)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_JP, 0x0820)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_JP_2, 0x0821)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_JP_1, 0x0822)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_JP_3, 0x0823)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_CN, 0x0830)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_CN_EXT, 0x0831)
        self.assertEqual(CoreFoundation.kCFStringEncodingISO_2022_KR, 0x0840)
        self.assertEqual(CoreFoundation.kCFStringEncodingEUC_JP, 0x0920)
        self.assertEqual(CoreFoundation.kCFStringEncodingEUC_CN, 0x0930)
        self.assertEqual(CoreFoundation.kCFStringEncodingEUC_TW, 0x0931)
        self.assertEqual(CoreFoundation.kCFStringEncodingEUC_KR, 0x0940)
        self.assertEqual(CoreFoundation.kCFStringEncodingShiftJIS, 0x0A01)
        self.assertEqual(CoreFoundation.kCFStringEncodingKOI8_R, 0x0A02)
        self.assertEqual(CoreFoundation.kCFStringEncodingBig5, 0x0A03)
        self.assertEqual(CoreFoundation.kCFStringEncodingMacRomanLatin1, 0x0A04)
        self.assertEqual(CoreFoundation.kCFStringEncodingHZ_GB_2312, 0x0A05)
        self.assertEqual(CoreFoundation.kCFStringEncodingBig5_HKSCS_1999, 0x0A06)
        self.assertEqual(CoreFoundation.kCFStringEncodingVISCII, 0x0A07)
        self.assertEqual(CoreFoundation.kCFStringEncodingKOI8_U, 0x0A08)
        self.assertEqual(CoreFoundation.kCFStringEncodingBig5_E, 0x0A09)
        self.assertEqual(CoreFoundation.kCFStringEncodingNextStepJapanese, 0x0B02)
        self.assertEqual(CoreFoundation.kCFStringEncodingEBCDIC_US, 0x0C01)
        self.assertEqual(CoreFoundation.kCFStringEncodingEBCDIC_CP037, 0x0C02)
        self.assertEqual(CoreFoundation.kCFStringEncodingShiftJIS_X0213_00, 0x0628)

    @min_os_level("10.6")
    def testFunctions10_6(self):
        self.assertResultIsBOOL(CoreFoundation.CFStringIsSurrogateHighCharacter)
        self.assertTrue(CoreFoundation.CFStringIsSurrogateHighCharacter(chr(0xD800)))
        self.assertFalse(CoreFoundation.CFStringIsSurrogateHighCharacter(chr(0x0600)))
        self.assertTrue(CoreFoundation.CFStringIsSurrogateLowCharacter(chr(0xDC00)))
        self.assertFalse(CoreFoundation.CFStringIsSurrogateLowCharacter(chr(0x0600)))
        v = CoreFoundation.CFStringGetLongCharacterForSurrogatePair(
            chr(0xD801), chr(0xDC01)
        )
        # self.assertEqual(v, ((1 << 10) | 1) + 0x0010000)
        self.assertEqual(v, 66561)

        self.assertResultIsBOOL(CoreFoundation.CFStringGetSurrogatePairForLongCharacter)
        ok, chars = CoreFoundation.CFStringGetSurrogatePairForLongCharacter(v, None)
        self.assertTrue(ok)

        if sys.version_info[:2] < (3, 3) == 65535:
            # ucs2 build of python 3.2 or earlier:
            self.assertEqual(len(chars), 2)
            self.assertEqual(chars[0], chr(0xD801))
            self.assertEqual(chars[1], chr(0xDC01))

        else:
            # ucs4 build of python 3.2 or earlier; or
            # python 3.3
            #
            # In both cases this function is useless because
            # Python can represent str codepoints without using
            # surrogate pairs, and will do so when converting
            # an array of UCS2 codepoints to a Pytho str object
            pass

    @min_os_level("10.7")
    def testFunctions10_7(self):
        loc = CoreFoundation.CFLocaleCopyCurrent()

        self.assertArgIsOut(CoreFoundation.CFStringGetHyphenationLocationBeforeIndex, 5)
        v, ch = CoreFoundation.CFStringGetHyphenationLocationBeforeIndex(
            "hello world", 5, CoreFoundation.CFRange(0, 10), 0, loc, None
        )
        self.assertIsInstance(v, int)
        self.assertIsInstance(ch, int)

        self.assertResultIsBOOL(CoreFoundation.CFStringIsHyphenationAvailableForLocale)
        v = CoreFoundation.CFStringIsHyphenationAvailableForLocale(loc)
        self.assertIsInstance(v, bool)



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class FindHelper(AppKit.NSObject):
    def rectsForCharacterRange_(self, r):
        return 1

    def replaceCharactersInRange_withString_(self, r, s):
        pass

    def stringLength(self):
        return 0

    def isSelectable(self):
        pass

    def allowsMultipleSelection(self):
        pass

    def isEditable(self):
        pass

    def stringAtIndex_effectiveRange_endswithSearchBoundary_(self, a, b, c):
        pass

    def firstSelectedRange(self):
        pass

    def scrollRangeToVisible_(self, a):
        pass

    def shouldReplaceCharactersInRanges_withStrings_(self, a, b):
        pass

    def contentViewAtIndex_effectiveCharacterRange_(self, a, b):
        pass

    def drawCharactersInRange_forContentView_(self, a, b):
        pass

    def isFindBarVisible(self):
        return 1

    def setFindBarVisible_(self, a):
        pass


class TestNSTextFinder(TestCase):
    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertEqual(AppKit.NSTextFinderActionShowFindInterface, 1)
        self.assertEqual(AppKit.NSTextFinderActionNextMatch, 2)
        self.assertEqual(AppKit.NSTextFinderActionPreviousMatch, 3)
        self.assertEqual(AppKit.NSTextFinderActionReplaceAll, 4)
        self.assertEqual(AppKit.NSTextFinderActionReplace, 5)
        self.assertEqual(AppKit.NSTextFinderActionReplaceAndFind, 6)
        self.assertEqual(AppKit.NSTextFinderActionSetSearchString, 7)
        self.assertEqual(AppKit.NSTextFinderActionReplaceAllInSelection, 8)
        self.assertEqual(AppKit.NSTextFinderActionSelectAll, 9)
        self.assertEqual(AppKit.NSTextFinderActionSelectAllInSelection, 10)
        self.assertEqual(AppKit.NSTextFinderActionHideFindInterface, 11)
        self.assertEqual(AppKit.NSTextFinderActionShowReplaceInterface, 12)
        self.assertEqual(AppKit.NSTextFinderActionHideReplaceInterface, 13)

        self.assertIsInstance(AppKit.NSTextFinderCaseInsensitiveKey, str)
        self.assertIsInstance(AppKit.NSTextFinderMatchingTypeKey, str)

        self.assertEqual(AppKit.NSTextFinderMatchingTypeContains, 0)
        self.assertEqual(AppKit.NSTextFinderMatchingTypeStartsWith, 1)
        self.assertEqual(AppKit.NSTextFinderMatchingTypeFullWord, 2)
        self.assertEqual(AppKit.NSTextFinderMatchingTypeEndsWith, 3)

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(AppKit.NSTextFinder.validateAction_)
        self.assertResultIsBOOL(AppKit.NSTextFinder.findIndicatorNeedsUpdate)
        self.assertArgIsBOOL(AppKit.NSTextFinder.setFindIndicatorNeedsUpdate_, 0)
        self.assertResultIsBOOL(AppKit.NSTextFinder.isIncrementalSearchingEnabled)
        self.assertArgIsBOOL(AppKit.NSTextFinder.setIncrementalSearchingEnabled_, 0)
        self.assertResultIsBOOL(
            AppKit.NSTextFinder.incrementalSearchingShouldDimContentView
        )
        self.assertArgIsBOOL(
            AppKit.NSTextFinder.setIncrementalSearchingShouldDimContentView_, 0
        )

    @min_os_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSTextFinderClient")

    @min_os_level("10.7")
    def testProtocol10_7(self):
        self.assertResultIsBOOL(FindHelper.isSelectable)
        self.assertResultIsBOOL(FindHelper.allowsMultipleSelection)
        self.assertResultIsBOOL(FindHelper.isEditable)

        self.assertArgHasType(
            FindHelper.stringAtIndex_effectiveRange_endswithSearchBoundary_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            FindHelper.stringAtIndex_effectiveRange_endswithSearchBoundary_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            FindHelper.stringAtIndex_effectiveRange_endswithSearchBoundary_,
            2,
            b"o^" + objc._C_NSBOOL,
        )

        self.assertResultHasType(FindHelper.stringLength, objc._C_NSUInteger)

        self.assertResultHasType(
            FindHelper.firstSelectedRange, AppKit.NSRange.__typestr__
        )
        self.assertArgHasType(
            FindHelper.scrollRangeToVisible_, 0, AppKit.NSRange.__typestr__
        )
        self.assertResultIsBOOL(FindHelper.shouldReplaceCharactersInRanges_withStrings_)
        self.assertArgHasType(
            FindHelper.replaceCharactersInRange_withString_,
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            FindHelper.rectsForCharacterRange_, 0, AppKit.NSRange.__typestr__
        )
        self.assertArgHasType(
            FindHelper.contentViewAtIndex_effectiveCharacterRange_,
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            FindHelper.contentViewAtIndex_effectiveCharacterRange_,
            1,
            b"o^" + AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            FindHelper.drawCharactersInRange_forContentView_,
            0,
            AppKit.NSRange.__typestr__,
        )

        objc.protocolNamed("NSTextFinderBarContainer")

        self.assertResultIsBOOL(FindHelper.isFindBarVisible)
        self.assertArgIsBOOL(FindHelper.setFindBarVisible_, 0)



# Place holder for real tests.
import objc
import Foundation
from PyObjCTools.TestSupport import TestCase


class TestNSLocalizedString(TestCase):
    def testBasic(self):
        # This is mostly a regression tests, the function used to crash on
        # this...
        if objc.platform != "MACOSX":
            return

        pool = Foundation.NSAutoreleasePool.alloc().init()
        s = Foundation.NSLocalizedString("hello world", "")
        del pool
        self.assertEqual(s, "hello world")
        # XXX : Since we get the same object back, it's still str
        # self.assertEqual (s.nsstring().description(), "hello world")

        pool = Foundation.NSAutoreleasePool.alloc().init()
        s = Foundation.NSLocalizedStringFromTable("hello world", "tab", "")
        del pool
        self.assertEqual(s, "hello world")

        pool = Foundation.NSAutoreleasePool.alloc().init()
        s = Foundation.NSLocalizedStringFromTableInBundle(
            "hello world", "tab", Foundation.NSBundle.mainBundle(), ""
        )
        del pool
        self.assertEqual(s, "hello world")

        pool = Foundation.NSAutoreleasePool.alloc().init()
        s = Foundation.NSLocalizedStringWithDefaultValue(
            "hello world", "tab", Foundation.NSBundle.mainBundle(), "default", ""
        )
        del pool
        self.assertEqual(s, "default")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSPageControllerHelper(AppKit.NSObject):
    def pageController_frameForObject_(self, p, f):
        return 1


class TestNSPageController(TestCase):
    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertEqual(AppKit.NSPageControllerTransitionStyleStackHistory, 0)
        self.assertEqual(AppKit.NSPageControllerTransitionStyleStackBook, 1)
        self.assertEqual(AppKit.NSPageControllerTransitionStyleHorizontalStrip, 2)

    def testProtocols(self):

        self.assertResultHasType(
            TestNSPageControllerHelper.pageController_frameForObject_,
            AppKit.NSRect.__typestr__,
        )

    @min_sdk_level("10.10")
    def testProtocolObjects(self):
        objc.protocolNamed("NSPageControllerDelegate")



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSRuleEditorHelper(AppKit.NSObject):
    def ruleEditor_numberOfChildrenForCriterion_withRowType_(self, ed, cr, rt):
        return 1

    def ruleEditor_child_forCriterion_withRowType_(self, ed, ch, cr, rt):
        return 1

    def ruleEditor_displayValueForCriterion_inRow_(self, ed, cr, rw):
        return 1

    def ruleEditor_predicatePartsForCriterion_withDisplayValue_inRow_(
        self, ed, cr, dv, rw
    ):
        return 1


class TestNSRuleEditor(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSRuleEditorNestingModeSingle, 0)
        self.assertEqual(AppKit.NSRuleEditorNestingModeList, 1)
        self.assertEqual(AppKit.NSRuleEditorNestingModeCompound, 2)
        self.assertEqual(AppKit.NSRuleEditorNestingModeSimple, 3)

        self.assertEqual(AppKit.NSRuleEditorRowTypeSimple, 0)
        self.assertEqual(AppKit.NSRuleEditorRowTypeCompound, 1)

        self.assertIsInstance(AppKit.NSRuleEditorPredicateLeftExpression, str)
        self.assertIsInstance(AppKit.NSRuleEditorPredicateRightExpression, str)
        self.assertIsInstance(AppKit.NSRuleEditorPredicateComparisonModifier, str)
        self.assertIsInstance(AppKit.NSRuleEditorPredicateOptions, str)
        self.assertIsInstance(AppKit.NSRuleEditorPredicateOperatorType, str)
        self.assertIsInstance(AppKit.NSRuleEditorPredicateCustomSelector, str)

        self.assertIsInstance(AppKit.NSRuleEditorPredicateCompoundType, str)
        self.assertIsInstance(AppKit.NSRuleEditorRowsDidChangeNotification, str)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSRuleEditor.isEditable)
        self.assertArgIsBOOL(AppKit.NSRuleEditor.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSRuleEditor.canRemoveAllRows)
        self.assertArgIsBOOL(AppKit.NSRuleEditor.setCanRemoveAllRows_, 0)
        self.assertArgIsBOOL(
            AppKit.NSRuleEditor.insertRowAtIndex_withType_asSubrowOfRow_animate_, 3
        )
        self.assertArgIsBOOL(AppKit.NSRuleEditor.removeRowsAtIndexes_includeSubrows_, 1)
        self.assertArgIsBOOL(
            AppKit.NSRuleEditor.selectRowIndexes_byExtendingSelection_, 1
        )

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSRuleEditorDelegate")

    def testProtocols(self):
        self.assertResultHasType(
            TestNSRuleEditorHelper.ruleEditor_numberOfChildrenForCriterion_withRowType_,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSRuleEditorHelper.ruleEditor_numberOfChildrenForCriterion_withRowType_,
            2,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSRuleEditorHelper.ruleEditor_child_forCriterion_withRowType_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSRuleEditorHelper.ruleEditor_child_forCriterion_withRowType_,
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSRuleEditorHelper.ruleEditor_displayValueForCriterion_inRow_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSRuleEditorHelper.ruleEditor_predicatePartsForCriterion_withDisplayValue_inRow_,  # noqa: B950
            3,
            objc._C_NSInteger,
        )



import CoreFoundation
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestBase(TestCase):
    def testConstants(self):
        self.assertEqual(CoreFoundation.TRUE, 1)
        self.assertEqual(CoreFoundation.FALSE, 0)

        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_0, 196.40)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_0_3, 196.50)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_1, 226.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_1_1, 226.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_1_2, 227.20)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_1_3, 227.20)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_1_4, 227.30)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2, 263.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_1, 263.10)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_2, 263.10)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_3, 263.30)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_4, 263.30)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_5, 263.50)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_6, 263.50)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_7, 263.50)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_2_8, 263.50)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3, 299.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_1, 299.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_2, 299.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_3, 299.30)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_4, 299.31)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_5, 299.31)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_6, 299.32)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_7, 299.33)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_8, 299.33)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_3_9, 299.35)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4, 368.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_1, 368.10)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_2, 368.11)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_3, 368.18)
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_4_Intel, 368.26
        )
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_4_PowerPC, 368.25
        )
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_5_Intel, 368.26
        )
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_5_PowerPC, 368.25
        )
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_6_Intel, 368.26
        )
        self.assertEqual(
            CoreFoundation.kCFCoreFoundationVersionNumber10_4_6_PowerPC, 368.25
        )
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_7, 368.27)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_8, 368.27)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_9, 368.28)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_10, 368.28)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_4_11, 368.31)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5, 476.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_1, 476.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_2, 476.10)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_3, 476.13)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_4, 476.14)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_5, 476.15)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_6, 476.17)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_7, 476.18)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_8, 476.19)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6, 550.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_1, 550.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_2, 550.13)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_3, 550.19)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_4, 550.29)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_5, 550.42)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_6, 550.42)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_7, 550.42)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_6_8, 550.43)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7, 635.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7_1, 635.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7_2, 635.15)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7_3, 635.19)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7_4, 635.21)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_7_5, 635.21)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_8, 744.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_8_1, 744.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_8_2, 744.12)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_8_3, 744.18)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_8_4, 744.19)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_9, 855.11)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_9_1, 855.11)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_9_2, 855.14)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10, 1151.16)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_1, 1151.16)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_2, 1152)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_3, 1153.18)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_4, 1153.18)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_5, 1153.18)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_10_Max, 1199)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11, 1253)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11_1, 1255.1)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11_2, 1256.14)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11_3, 1256.14)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11_4, 1258.1)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_11_Max, 1299)

        self.assertEqual(CoreFoundation.kCFCompareLessThan, -1)
        self.assertEqual(CoreFoundation.kCFCompareEqualTo, 0)
        self.assertEqual(CoreFoundation.kCFCompareGreaterThan, 1)

        self.assertEqual(CoreFoundation.kCFNotFound, -1)

    def testStructs(self):

        o = CoreFoundation.CFRange()
        self.assertHasAttr(o, "location")
        self.assertHasAttr(o, "length")

    def testCFRangeMake(self):
        r = CoreFoundation.CFRangeMake(42, 99)
        self.assertIsInstance(r, CoreFoundation.CFRange)
        self.assertEqual(r.location, 42)
        self.assertEqual(r.length, 99)

    def testCFNull(self):
        self.assertIsInstance(CoreFoundation.CFNullGetTypeID(), int)
        self.assertIsInstance(CoreFoundation.kCFNull, CoreFoundation.CFNullRef)

        cls = objc.lookUpClass("NSNull")
        if cls is not CoreFoundation.CFNullRef:
            self.assertIsCFType(CoreFoundation.CFNullRef)

    def testCFAllocator(self):
        self.assertIsCFType(CoreFoundation.CFAllocatorRef)

        self.assertIsInstance(CoreFoundation.CFAllocatorGetTypeID(), int)
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorDefault,
            (CoreFoundation.CFAllocatorRef, type(None)),
        )
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorSystemDefault, CoreFoundation.CFAllocatorRef
        )
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorMalloc, CoreFoundation.CFAllocatorRef
        )
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorMallocZone, CoreFoundation.CFAllocatorRef
        )
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorNull, CoreFoundation.CFAllocatorRef
        )
        self.assertIsInstance(
            CoreFoundation.kCFAllocatorUseContext, CoreFoundation.CFAllocatorRef
        )
        r = CoreFoundation.CFAllocatorGetDefault()
        self.assertIsInstance(r, CoreFoundation.CFAllocatorRef)
        CoreFoundation.CFAllocatorSetDefault(CoreFoundation.kCFAllocatorMalloc)
        r2 = CoreFoundation.CFAllocatorGetDefault()
        self.assertIsInstance(r2, CoreFoundation.CFAllocatorRef)
        self.assertIs(r2, CoreFoundation.kCFAllocatorMalloc)
        # Restore default allocator
        CoreFoundation.CFAllocatorSetDefault(r)

        self.assertNotHasAttr(CoreFoundation, "CFAllocatorCreate")
        self.assertNotHasAttr(CoreFoundation, "CFAllocatorAllocate")
        self.assertNotHasAttr(CoreFoundation, "CFAllocatorReallocate")
        self.assertNotHasAttr(CoreFoundation, "CFAllocatorDeallocate")
        self.assertNotHasAttr(CoreFoundation, "CFAllocatorGetContext")
        r = CoreFoundation.CFAllocatorGetPreferredSizeForSize(
            CoreFoundation.kCFAllocatorDefault, 15, 0
        )
        self.assertIsInstance(r, int)
        self.assertGreaterEqual(r, 15)

    def testGenericFunctions(self):
        value = CoreFoundation.CFGetTypeID(CoreFoundation.kCFAllocatorMalloc)
        self.assertEqual(value, CoreFoundation.CFAllocatorGetTypeID())

        v = CoreFoundation.CFCopyTypeIDDescription(
            CoreFoundation.CFAllocatorGetTypeID()
        )
        self.assertIsInstance(v, str)
        obj = CoreFoundation.CFURLCreateWithString(None, "http://www.apple.com/", None)
        i = CoreFoundation.CFGetTypeID(obj)
        self.assertIsInstance(i, int)
        i = CoreFoundation.CFGetRetainCount(obj)
        self.assertIsInstance(i, int)
        CoreFoundation.CFRetain(obj)
        self.assertEqual(CoreFoundation.CFGetRetainCount(obj), i + 1)
        CoreFoundation.CFRelease(obj)
        self.assertEqual(CoreFoundation.CFGetRetainCount(obj), i)

        CoreFoundation.CFRetain(obj)
        CoreFoundation.CFMakeCollectable(obj)

        del obj

        i = CoreFoundation.CFEqual(
            CoreFoundation.kCFAllocatorMalloc, CoreFoundation.kCFAllocatorNull
        )
        self.assertIs(i, False)
        i = CoreFoundation.CFEqual(
            CoreFoundation.kCFAllocatorMalloc, CoreFoundation.kCFAllocatorMalloc
        )
        self.assertIs(i, True)
        i = CoreFoundation.CFHash(CoreFoundation.kCFAllocatorMalloc)
        self.assertTrue(i, int)

        v = CoreFoundation.CFCopyDescription(CoreFoundation.kCFAllocatorMalloc)
        self.assertIsInstance(v, str)
        v = CoreFoundation.CFGetAllocator(CoreFoundation.kCFAllocatorMalloc)
        if v is not None:
            self.assertIsInstance(v, CoreFoundation.CFAllocatorRef)

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5, 476.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_1, 476.00)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_2, 476.10)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_3, 476.13)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_4, 476.14)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_5, 476.15)
        self.assertEqual(CoreFoundation.kCFCoreFoundationVersionNumber10_5_6, 476.17)

    @min_os_level("10.9")
    def testFunctions10_9(self):
        obj = CoreFoundation.CFURLCreateWithString(None, "http://www.apple.com/", None)
        CoreFoundation.CFRetain(obj)
        CoreFoundation.CFAutorelease(obj)
        del obj



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSBrowserHelper(AppKit.NSObject):
    def browser_namesOfPromisedFilesDroppedAtDestination_forDraggedRowsWithIndexes_inColumn_(
        self, b, n, i, c
    ):
        pass

    def browser_willDisplayCell_atRow_column_(self, b, cl, r, c):
        pass

    def browser_numberOfChildrenOfItem_(self, b, i):
        return 1

    def browser_child_ofItem_(self, b, c, i):
        return None

    def browser_selectCellWithString_inColumn_(self, b, c, s):
        return 1

    def browser_selectRow_inColumn_(self, b, r, c):
        return 1

    def browser_isColumnValid_(self, b, c):
        return 1

    def browser_shouldSizeColumn_forUserResize_toWidth_(self, b, c, us, w):
        return 1

    def browser_shouldShowCellExpansionForRow_column_(self, b, r, c):
        return 1

    def browser_writeRowsWithIndexes_inColumn_toPasteboard_(self, b, r, c, p):
        return 1

    def browser_canDragRowsWithIndexes_inColumn_withEvent_(self, b, i, c, e):
        return 1

    def browser_acceptDrop_atRow_column_dropOperation_(self, b, a, r, c, o):
        return 1

    def browser_shouldTypeSelectForEvent_withCurrentSearchString_(self, b, e, s):
        return 1

    def browser_isLeafItem_(self, b, s):
        return 1

    def browser_heightOfRow_inColumn_(self, b, r, c):
        return 1

    def browser_shouldEditItem_(self, b, i):
        return 1

    def browser_numberOfRowsInColumn_(self, b, c):
        return 1

    def browser_createRowsForColumn_inMatrix_(self, b, c, m):
        return 1

    def browser_selectionIndexesForProposedSelection_inColumn_(self, b, s, c):
        return 1

    def browser_sizeToFitWidthOfColumn_(self, b, c):
        return 1

    def browser_draggingImageForRowsWithIndexes_inColumn_withEvent_offset_(
        self, b, ixs, c, e, o
    ):
        return 1

    def browser_validateDrop_proposedRow_column_dropOperation_(self, b, dp, r, c, o):
        return 1

    def browser_typeSelectStringForRow_inColumn_(self, b, r, c):
        return 1

    def browser_nextTypeSelectMatchFromRow_toRow_inColumn_forString_(
        self, b, r, r2, c, s
    ):
        return 1

    def browser_didChangeLastColumn_toColumn_(self, b, c1, c2):
        pass


class TestNSBrowser(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSBrowserNoColumnResizing, 0)
        self.assertEqual(AppKit.NSBrowserAutoColumnResizing, 1)
        self.assertEqual(AppKit.NSBrowserUserColumnResizing, 2)

        self.assertEqual(AppKit.NSBrowserDropOn, 0)
        self.assertEqual(AppKit.NSBrowserDropAbove, 1)

        self.assertIsInstance(
            AppKit.NSBrowserColumnConfigurationDidChangeNotification, str
        )
        self.assertEqual(
            AppKit.NSAppKitVersionNumberWithContinuousScrollingBrowser, 680.0
        )
        self.assertEqual(AppKit.NSAppKitVersionNumberWithColumnResizingBrowser, 685.0)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSBrowser.isLoaded)
        self.assertResultIsBOOL(AppKit.NSBrowser.reusesColumns)
        self.assertArgIsBOOL(AppKit.NSBrowser.setReusesColumns_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.hasHorizontalScroller)
        self.assertArgIsBOOL(AppKit.NSBrowser.setHasHorizontalScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.separatesColumns)
        self.assertArgIsBOOL(AppKit.NSBrowser.setSeparatesColumns_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.isTitled)
        self.assertArgIsBOOL(AppKit.NSBrowser.setTitled_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.allowsMultipleSelection)
        self.assertArgIsBOOL(AppKit.NSBrowser.setAllowsMultipleSelection_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.allowsBranchSelection)
        self.assertArgIsBOOL(AppKit.NSBrowser.setAllowsBranchSelection_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.allowsEmptySelection)
        self.assertArgIsBOOL(AppKit.NSBrowser.setAllowsEmptySelection_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.takesTitleFromPreviousColumn)
        self.assertArgIsBOOL(AppKit.NSBrowser.setTakesTitleFromPreviousColumn_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.acceptsArrowKeys)
        self.assertArgIsBOOL(AppKit.NSBrowser.setAcceptsArrowKeys_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.sendsActionOnArrowKeys)
        self.assertArgIsBOOL(AppKit.NSBrowser.setSendsActionOnArrowKeys_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.setPath_)
        self.assertResultIsBOOL(AppKit.NSBrowser.sendAction)
        self.assertResultIsBOOL(AppKit.NSBrowser.prefersAllColumnUserResizing)
        self.assertArgIsBOOL(AppKit.NSBrowser.setPrefersAllColumnUserResizing_, 0)
        self.assertResultIsBOOL(
            AppKit.NSBrowser.canDragRowsWithIndexes_inColumn_withEvent_
        )
        self.assertArgIsBOOL(
            AppKit.NSBrowser.setDraggingSourceOperationMask_forLocal_, 1
        )
        self.assertResultIsBOOL(AppKit.NSBrowser.allowsTypeSelect)
        self.assertArgIsBOOL(AppKit.NSBrowser.setAllowsTypeSelect_, 0)

    @min_sdk_level("10.6")
    def testProtocolObjects(self):
        objc.protocolNamed("NSBrowserDelegate")

    def testDelegate(self):
        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_selectCellWithString_inColumn_
        )
        self.assertResultIsBOOL(TestNSBrowserHelper.browser_selectRow_inColumn_)
        self.assertResultIsBOOL(TestNSBrowserHelper.browser_isColumnValid_)
        self.assertArgIsBOOL(
            TestNSBrowserHelper.browser_shouldSizeColumn_forUserResize_toWidth_, 2
        )
        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_shouldShowCellExpansionForRow_column_
        )
        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_writeRowsWithIndexes_inColumn_toPasteboard_
        )
        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_canDragRowsWithIndexes_inColumn_withEvent_
        )
        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_acceptDrop_atRow_column_dropOperation_
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_namesOfPromisedFilesDroppedAtDestination_forDraggedRowsWithIndexes_inColumn_,  # noqa: B950
            3,
            objc._C_NSInteger,
        )

        self.assertResultIsBOOL(
            TestNSBrowserHelper.browser_shouldTypeSelectForEvent_withCurrentSearchString_
        )
        self.assertResultHasType(
            TestNSBrowserHelper.browser_sizeToFitWidthOfColumn_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_sizeToFitWidthOfColumn_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_draggingImageForRowsWithIndexes_inColumn_withEvent_offset_,  # noqa: B950
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_draggingImageForRowsWithIndexes_inColumn_withEvent_offset_,  # noqa: B950
            4,
            b"N^" + AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_validateDrop_proposedRow_column_dropOperation_,
            2,
            b"N^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_validateDrop_proposedRow_column_dropOperation_,
            3,
            b"N^" + objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_validateDrop_proposedRow_column_dropOperation_,
            4,
            b"N^" + objc._C_NSUInteger,
        )
        self.assertResultHasType(
            TestNSBrowserHelper.browser_validateDrop_proposedRow_column_dropOperation_,
            objc._C_NSUInteger,
        )

        self.assertArgHasType(
            TestNSBrowserHelper.browser_typeSelectStringForRow_inColumn_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_typeSelectStringForRow_inColumn_,
            2,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSBrowserHelper.browser_nextTypeSelectMatchFromRow_toRow_inColumn_forString_,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_nextTypeSelectMatchFromRow_toRow_inColumn_forString_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_nextTypeSelectMatchFromRow_toRow_inColumn_forString_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_nextTypeSelectMatchFromRow_toRow_inColumn_forString_,
            3,
            objc._C_NSInteger,
        )

        self.assertArgHasType(
            TestNSBrowserHelper.browser_didChangeLastColumn_toColumn_,
            1,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_didChangeLastColumn_toColumn_,
            2,
            objc._C_NSInteger,
        )

        # XXX: Redo testcase for delegate

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBOOL(AppKit.NSBrowser.setAutohidesScroller_, 0)
        self.assertResultIsBOOL(AppKit.NSBrowser.autohidesScroller)
        self.assertResultIsBOOL(AppKit.NSBrowser.isLeafItem_)
        self.assertResultIsBOOL(AppKit.NSBrowser.getRow_column_forPoint_)
        self.assertArgIsOut(AppKit.NSBrowser.getRow_column_forPoint_, 0)
        self.assertArgIsOut(AppKit.NSBrowser.getRow_column_forPoint_, 1)
        self.assertArgIsBOOL(AppKit.NSBrowser.editItemAtIndexPath_withEvent_select_, 2)

        self.assertArgHasType(
            TestNSBrowserHelper.browser_selectionIndexesForProposedSelection_inColumn_,
            2,
            objc._C_NSInteger,
        )

    @min_os_level("10.6")
    def testDelegate10_6(self):
        self.assertResultIsBOOL(TestNSBrowserHelper.browser_isLeafItem_)
        self.assertResultHasType(
            TestNSBrowserHelper.browser_heightOfRow_inColumn_, objc._C_CGFloat
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_heightOfRow_inColumn_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_heightOfRow_inColumn_, 2, objc._C_NSInteger
        )
        self.assertResultIsBOOL(TestNSBrowserHelper.browser_shouldEditItem_)
        self.assertResultHasType(
            TestNSBrowserHelper.browser_numberOfRowsInColumn_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_numberOfRowsInColumn_, 1, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_createRowsForColumn_inMatrix_,
            1,
            objc._C_NSInteger,
        )

        self.assertResultHasType(
            TestNSBrowserHelper.browser_numberOfChildrenOfItem_, objc._C_NSInteger
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_child_ofItem_, 1, objc._C_NSInteger
        )

        self.assertArgHasType(
            TestNSBrowserHelper.browser_willDisplayCell_atRow_column_,
            2,
            objc._C_NSInteger,
        )
        self.assertArgHasType(
            TestNSBrowserHelper.browser_willDisplayCell_atRow_column_,
            3,
            objc._C_NSInteger,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_sdk_level
import objc


class TestNSFilePromiseProviderHelper(AppKit.NSObject):
    def filePromiseProvider_writePromiseToURL_completionHandler_(self, p, u, h):
        pass


class TestNSFilePromiseProvider(TestCase):
    @min_sdk_level("10.12")
    def testProtocols(self):
        objc.protocolNamed("NSFilePromiseProviderDelegate")

    def testMethods(self):
        self.assertArgIsBlock(
            TestNSFilePromiseProviderHelper.filePromiseProvider_writePromiseToURL_completionHandler_,  # noqa: B950
            2,
            b"v@",
        )



import objc


def __len__(self):
    return self.length()


def __getitem__(self, idx):
    if isinstance(idx, slice):
        raise ValueError(idx)
    return self.indexAtPosition_(idx)


def __add__(self, value):
    return self.indexPathByAddingIndex_(value)


objc.addConvenienceForClass(
    "NSIndexPath",
    (("__len__", __len__), ("__getitem__", __getitem__), ("__add__", __add__)),
)



import AppKit
from PyObjCTools.TestSupport import TestCase


class TestNSObjectController(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSObjectController.automaticallyPreparesContent)
        self.assertArgIsBOOL(
            AppKit.NSObjectController.setAutomaticallyPreparesContent_, 0
        )
        self.assertResultIsBOOL(AppKit.NSObjectController.isEditable)
        self.assertArgIsBOOL(AppKit.NSObjectController.setEditable_, 0)
        self.assertResultIsBOOL(AppKit.NSObjectController.canAdd)
        self.assertResultIsBOOL(AppKit.NSObjectController.canRemove)
        self.assertResultIsBOOL(AppKit.NSObjectController.validateUserInterfaceItem_)
        self.assertResultIsBOOL(AppKit.NSObjectController.fetchWithRequest_merge_error_)
        self.assertArgIsBOOL(AppKit.NSObjectController.fetchWithRequest_merge_error_, 1)
        self.assertArgIsOut(AppKit.NSObjectController.fetchWithRequest_merge_error_, 2)
        self.assertResultIsBOOL(AppKit.NSObjectController.usesLazyFetching)
        self.assertArgIsBOOL(AppKit.NSObjectController.setUsesLazyFetching_, 0)



import CoreFoundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure


class TestCFFileSecurity(TestCase):
    @min_os_level("10.7")
    def testTypes(self):
        try:
            cls = objc.lookUpClass("__NSFileSecurity")
            self.assertIs(cls, CoreFoundation.CFFileSecurityRef)
        except objc.error:
            self.assertIsCFType(CoreFoundation.CFFileSecurityRef)

    @min_os_level("10.7")
    def testTypeID(self):
        self.assertIsInstance(CoreFoundation.CFFileSecurityGetTypeID(), int)

    @min_os_level("10.7")
    @expectedFailure
    def testConstants(self):
        self.fail("kCFFileSecurityRemoveACL")

    @min_os_level("10.8")
    @expectedFailure
    def testConstants10_8(self):
        self.assertEqual(CoreFoundation.kCFFileSecurityClearOwner, 1 << 0)
        self.assertEqual(CoreFoundation.kCFFileSecurityClearGroup, 1 << 1)
        self.assertEqual(CoreFoundation.kCFFileSecurityClearMode, 1 << 2)
        self.assertEqual(CoreFoundation.kCFFileSecurityClearOwnerUUID, 1 << 3)
        self.assertEqual(CoreFoundation.kCFFileSecurityClearGroupUUID, 1 << 4)
        self.assertEqual(CoreFoundation.kCFFileSecurityClearAccessControlList, 1 << 5)

    @min_os_level("10.7")
    def testFunctions10_7(self):
        self.assertResultIsCFRetained(CoreFoundation.CFFileSecurityCreate)
        v = CoreFoundation.CFFileSecurityCreate(None)
        self.assertIsInstance(v, CoreFoundation.CFFileSecurityRef)

        self.assertResultIsCFRetained(CoreFoundation.CFFileSecurityCreateCopy)
        o = CoreFoundation.CFFileSecurityCreateCopy(None, v)
        self.assertIsInstance(o, CoreFoundation.CFFileSecurityRef)

        self.assertResultIsBOOL(CoreFoundation.CFFileSecurityCopyOwnerUUID)
        self.assertArgIsOut(CoreFoundation.CFFileSecurityCopyOwnerUUID, 1)
        self.assertResultIsBOOL(CoreFoundation.CFFileSecuritySetOwnerUUID)

        self.assertResultIsBOOL(CoreFoundation.CFFileSecurityCopyGroupUUID)
        self.assertArgIsOut(CoreFoundation.CFFileSecurityCopyGroupUUID, 1)
        self.assertResultIsBOOL(CoreFoundation.CFFileSecuritySetGroupUUID)

        self.assertResultIsBOOL(CoreFoundation.CFFileSecurityGetOwner)
        self.assertArgIsOut(CoreFoundation.CFFileSecurityGetOwner, 1)
        self.assertResultIsBOOL(CoreFoundation.CFFileSecuritySetOwner)

        self.assertResultIsBOOL(CoreFoundation.CFFileSecurityGetGroup)
        self.assertArgIsOut(CoreFoundation.CFFileSecurityGetGroup, 1)
        self.assertResultIsBOOL(CoreFoundation.CFFileSecuritySetGroup)

        self.assertResultIsBOOL(CoreFoundation.CFFileSecurityGetMode)
        self.assertArgIsOut(CoreFoundation.CFFileSecurityGetMode, 1)
        self.assertResultIsBOOL(CoreFoundation.CFFileSecuritySetMode)

        security = CoreFoundation.CFFileSecurityCreate(None)
        self.assertIsInstance(v, CoreFoundation.CFFileSecurityRef)

        ok = CoreFoundation.CFFileSecuritySetOwner(security, 44)
        self.assertTrue(ok)
        ok, v = CoreFoundation.CFFileSecurityGetOwner(security, None)
        self.assertTrue(ok)
        self.assertEqual(v, 44)

        ok = CoreFoundation.CFFileSecuritySetGroup(security, 999)
        self.assertTrue(ok)
        ok, v = CoreFoundation.CFFileSecurityGetGroup(security, None)
        self.assertTrue(ok)
        self.assertEqual(v, 999)

        ok = CoreFoundation.CFFileSecuritySetMode(security, 0o444)
        self.assertTrue(ok)
        ok, v = CoreFoundation.CFFileSecurityGetMode(security, None)
        self.assertTrue(ok)
        self.assertEqual(v, 0o444)

    @min_os_level("10.7")
    @expectedFailure
    def testFunctionsUnwrapped(self):
        # There are no usable wrappers for sys/acl.h at this time
        self.fail("ACL Handling")

    @min_os_level("10.8")
    def testFunctions10_8(self):
        security = CoreFoundation.CFFileSecurityCreate(None)
        self.assertIsInstance(security, CoreFoundation.CFFileSecurityRef)

        CoreFoundation.CFFileSecurityClearProperties(
            security, CoreFoundation.kCFFileSecurityClearGroup
        )

    @min_os_level("10.7")
    def testFunctions_unsupported(self):
        self.assertFalse(hasattr(CoreFoundation, "CFFileSecurityCopyAccessControlList"))
        self.assertFalse(hasattr(CoreFoundation, "CFFileSecuritySetAccessControlList"))



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSpellChecker(TestCase):
    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSpellChecker.sharedSpellCheckerExists)
        self.assertArgIsBOOL(
            AppKit.NSSpellChecker.checkSpellingOfString_startingAt_language_wrap_inSpellDocumentWithTag_wordCount_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSSpellChecker.checkSpellingOfString_startingAt_language_wrap_inSpellDocumentWithTag_wordCount_,  # noqa: B950
            5,
        )
        self.assertResultIsBOOL(AppKit.NSSpellChecker.setLanguage_)

        self.assertArgHasType(
            AppKit.NSSpellChecker.completionsForPartialWordRange_inString_language_inSpellDocumentWithTag_,  # noqa: B950
            0,
            AppKit.NSRange.__typestr__,
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgIsBOOL(
            AppKit.NSSpellChecker.checkGrammarOfString_startingAt_language_wrap_inSpellDocumentWithTag_details_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSSpellChecker.checkGrammarOfString_startingAt_language_wrap_inSpellDocumentWithTag_details_,  # noqa: B950
            5,
        )
        self.assertResultIsBOOL(AppKit.NSSpellChecker.hasLearnedWord_)

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgHasType(
            AppKit.NSSpellChecker.checkString_range_types_options_inSpellDocumentWithTag_orthography_wordCount_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgIsOut(
            AppKit.NSSpellChecker.checkString_range_types_options_inSpellDocumentWithTag_orthography_wordCount_,  # noqa: B950
            5,
        )
        self.assertArgIsOut(
            AppKit.NSSpellChecker.checkString_range_types_options_inSpellDocumentWithTag_orthography_wordCount_,  # noqa: B950
            6,
        )

        self.assertArgHasType(
            AppKit.NSSpellChecker.requestCheckingOfString_range_types_options_inSpellDocumentWithTag_completionHandler_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgIsBlock(
            AppKit.NSSpellChecker.requestCheckingOfString_range_types_options_inSpellDocumentWithTag_completionHandler_,  # noqa: B950
            5,
            b"v" + objc._C_NSInteger + b"@@" + objc._C_NSInteger,
        )

        self.assertArgHasType(
            AppKit.NSSpellChecker.menuForResult_string_options_atLocation_inView_,
            3,
            AppKit.NSPoint.__typestr__,
        )

        self.assertArgHasType(
            AppKit.NSSpellChecker.guessesForWordRange_inString_language_inSpellDocumentWithTag_,
            0,
            AppKit.NSRange.__typestr__,
        )

        self.assertResultIsBOOL(AppKit.NSSpellChecker.automaticallyIdentifiesLanguages)
        self.assertArgIsBOOL(
            AppKit.NSSpellChecker.setAutomaticallyIdentifiesLanguages_, 0
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertArgIsBlock(
            AppKit.NSSpellChecker.showCorrectionIndicatorOfType_primaryString_alternativeStrings_forStringInRect_view_completionHandler_,  # noqa: B950
            5,
            b"v@",
        )
        self.assertResultIsBOOL(AppKit.NSSpellChecker.isAutomaticTextReplacementEnabled)
        self.assertResultIsBOOL(
            AppKit.NSSpellChecker.isAutomaticSpellingCorrectionEnabled
        )

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertResultIsBOOL(
            AppKit.NSSpellChecker.isAutomaticQuoteSubstitutionEnabled
        )
        self.assertResultIsBOOL(
            AppKit.NSSpellChecker.isAutomaticDashSubstitutionEnabled
        )

    @min_os_level("10.12")
    def testMethods10_12(self):
        self.assertResultIsBOOL(
            AppKit.NSSpellChecker.preventsAutocorrectionBeforeString_language_
        )
        self.assertResultIsBOOL(AppKit.NSSpellChecker.isAutomaticCapitalizationEnabled)
        self.assertResultIsBOOL(
            AppKit.NSSpellChecker.isAutomaticPeriodSubstitutionEnabled
        )
        self.assertResultIsBOOL(AppKit.NSSpellChecker.isAutomaticTextCompletionEnabled)

        self.assertArgIsBlock(
            AppKit.NSSpellChecker.requestCandidatesForSelectedRange_inString_types_options_inSpellDocumentWithTag_completionHandler_,  # noqa: B950
            5,
            b"v" + objc._C_NSInteger + b"@",
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertIsInstance(AppKit.NSTextCheckingOrthographyKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingQuotesKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingReplacementsKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingReferenceDateKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingReferenceTimeZoneKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingDocumentURLKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingDocumentTitleKey, str)
        self.assertIsInstance(AppKit.NSTextCheckingDocumentAuthorKey, str)

    @min_os_level("10.7")
    def testConstants10_7(self):
        self.assertIsInstance(AppKit.NSTextCheckingRegularExpressionsKey, str)
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticSpellingCorrectionNotification, str
        )
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticTextReplacementNotification, str
        )

        self.assertEqual(AppKit.NSCorrectionResponseNone, 0)
        self.assertEqual(AppKit.NSCorrectionResponseAccepted, 1)
        self.assertEqual(AppKit.NSCorrectionResponseRejected, 2)
        self.assertEqual(AppKit.NSCorrectionResponseIgnored, 3)
        self.assertEqual(AppKit.NSCorrectionResponseEdited, 4)
        self.assertEqual(AppKit.NSCorrectionResponseReverted, 5)

        self.assertEqual(AppKit.NSCorrectionIndicatorTypeDefault, 0)
        self.assertEqual(AppKit.NSCorrectionIndicatorTypeReversion, 1)
        self.assertEqual(AppKit.NSCorrectionIndicatorTypeGuesses, 2)

    @min_os_level("10.9")
    def testConstants10_9(self):
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticQuoteSubstitutionNotification, str
        )
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticDashSubstitutionNotification, str
        )

    @min_os_level("10.12")
    def testConstants10_12(self):
        self.assertIsInstance(AppKit.NSTextCheckingSelectedRangeKey, str)
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticCapitalizationNotification, str
        )
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticPeriodSubstitutionNotification, str
        )
        self.assertIsInstance(
            AppKit.NSSpellCheckerDidChangeAutomaticTextCompletionNotification, str
        )



import AppKit
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSTypesetterHelper(AppKit.NSTypesetter):
    def willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_(
        self, a, b, c, d
    ):
        return 1

    def getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_(
        self, a, b, c, d, e, f
    ):
        return 1

    def getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_(  # noqa: B950
        self, a, b, c, d, e, f, g, h
    ):
        return 1

    def shouldBreakLineByWordBeforeCharacterAtIndex_(self, i):
        return False

    def shouldBreakLineByHyphenatingBeforeCharacterAtIndex_(self, i):
        return False

    def hyphenationFactorForGlyphAtIndex_(self, i):
        return 1.0

    def hyphenCharacterForGlyphAtIndex_(self, i):
        return 1

    def boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_(  # noqa: B950
        self, i, c, f, p, idx
    ):
        pass


class TestNSTypesetter(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTypesetterZeroAdvancementAction, (1 << 0))
        self.assertEqual(AppKit.NSTypesetterWhitespaceAction, (1 << 1))
        self.assertEqual(AppKit.NSTypesetterHorizontalTabAction, (1 << 2))
        self.assertEqual(AppKit.NSTypesetterLineBreakAction, (1 << 3))
        self.assertEqual(AppKit.NSTypesetterParagraphBreakAction, (1 << 4))
        self.assertEqual(AppKit.NSTypesetterContainerBreakAction, (1 << 5))

    def testMethods(self):
        self.assertArgIsIn(AppKit.NSTypesetter.substituteGlyphsInRange_withGlyphs_, 1)
        self.assertArgSizeInArg(
            AppKit.NSTypesetter.substituteGlyphsInRange_withGlyphs_, 1, 0
        )
        self.assertArgIsIn(
            AppKit.NSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_, 1
        )
        self.assertArgSizeInArg(
            AppKit.NSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_, 1, 2
        )
        self.assertArgIsBOOL(AppKit.NSTypesetter.setNotShownAttribute_forGlyphRange_, 0)
        self.assertArgIsBOOL(
            AppKit.NSTypesetter.setDrawsOutsideLineFragment_forGlyphRange_, 0
        )
        self.assertResultIsBOOL(
            TestNSTypesetterHelper.shouldBreakLineByWordBeforeCharacterAtIndex_
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.shouldBreakLineByWordBeforeCharacterAtIndex_,
            0,
            objc._C_NSUInteger,
        )

        self.assertResultIsBOOL(
            TestNSTypesetterHelper.shouldBreakLineByHyphenatingBeforeCharacterAtIndex_
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.shouldBreakLineByHyphenatingBeforeCharacterAtIndex_,
            0,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSTypesetterHelper.hyphenationFactorForGlyphAtIndex_, objc._C_FLT
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.hyphenationFactorForGlyphAtIndex_,
            0,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSTypesetterHelper.boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            0,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            2,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            3,
            AppKit.NSPoint.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.boundingBoxForControlGlyphAtIndex_forTextContainer_proposedLineFragment_glyphPosition_characterIndex_,  # noqa: B950
            4,
            objc._C_NSUInteger,
        )

        self.assertResultHasType(
            TestNSTypesetterHelper.hyphenCharacterForGlyphAtIndex_, objc._C_INT
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.hyphenCharacterForGlyphAtIndex_,
            0,
            objc._C_NSUInteger,
        )

        self.assertArgIsBOOL(AppKit.NSTypesetter.setHardInvalidation_forGlyphRange_, 0)
        self.assertResultIsBOOL(AppKit.NSTypesetter.usesFontLeading)
        self.assertArgIsBOOL(AppKit.NSTypesetter.setUsesFontLeading_, 0)
        self.assertResultIsBOOL(AppKit.NSTypesetter.bidiProcessingEnabled)
        self.assertArgIsBOOL(AppKit.NSTypesetter.setBidiProcessingEnabled_, 0)
        self.assertArgIsInOut(AppKit.NSTypesetter.layoutParagraphAtPoint_, 0)
        self.assertArgIsOut(
            AppKit.NSTypesetter.characterRangeForGlyphRange_actualGlyphRange_, 1
        )
        self.assertArgIsOut(
            AppKit.NSTypesetter.glyphRangeForCharacterRange_actualCharacterRange_, 1
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.setBidiLevels_forGlyphRange_,
            0,
            b"n^" + objc._C_CHAR_AS_INT,
        )
        self.assertArgSizeInArg(AppKit.NSTypesetter.setBidiLevels_forGlyphRange_, 0, 1)
        self.assertArgIsIn(
            AppKit.NSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_, 1
        )
        self.assertArgSizeInArg(
            AppKit.NSTypesetter.setLocation_withAdvancements_forStartOfGlyphRange_, 1, 2
        )
        self.assertArgIsOut(
            AppKit.NSTypesetter.layoutGlyphsInLayoutManager_startingAtGlyphIndex_maxNumberOfLineFragments_nextGlyphIndex_,  # noqa: B950
            3,
        )
        self.assertArgIsOut(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_forParagraphSeparatorGlyphRange_atProposedOrigin_,  # noqa: B950
            0,
        )
        self.assertArgIsOut(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_forParagraphSeparatorGlyphRange_atProposedOrigin_,  # noqa: B950
            1,
        )

        self.assertResultHasType(
            AppKit.NSTypesetter.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            0,
            AppKit.NSRange.__typestr__,
        )

        self.assertArgHasType(
            TestNSTypesetterHelper.willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_,  # noqa: B950
            0,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_,  # noqa: B950
            1,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_,  # noqa: B950
            2,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.willSetLineFragmentRect_forGlyphRange_usedRect_baselineOffset_,  # noqa: B950
            3,
            b"N^" + objc._C_CGFloat,
        )

        self.assertResultHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            0,
            AppKit.NSRange.__typestr__,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            1,
            b"o^I",
        )
        self.assertArgSizeInArg(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            1,
            0,
        )

        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            2,
            b"o^" + objc._C_NSUInteger,
        )
        self.assertArgSizeInArg(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            2,
            0,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            3,
            b"o^" + objc._C_NSUInteger,
        )
        self.assertArgSizeInArg(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            3,
            0,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            4,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertArgSizeInArg(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            4,
            0,
        )
        self.assertArgHasType(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            5,
            b"o^" + objc._C_UCHR,
        )
        self.assertArgSizeInArg(
            TestNSTypesetterHelper.getGlyphsInRange_glyphs_characterIndexes_glyphInscriptions_elasticBits_bidiLevels_,  # noqa: B950
            5,
            0,
        )

        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            0,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            1,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            2,
            b"N^" + AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            3,
            objc._C_NSUInteger,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            4,
            AppKit.NSRect.__typestr__,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            5,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            6,
            objc._C_CGFloat,
        )
        self.assertArgHasType(
            AppKit.NSTypesetter.getLineFragmentRect_usedRect_remainingRect_forStartingGlyphAtIndex_proposedRect_lineSpacing_paragraphSpacingBefore_paragraphSpacingAfter_,  # noqa: B950
            7,
            objc._C_CGFloat,
        )

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertArgHasType(
            AppKit.NSTypesetter.printingAdjustmentInLayoutManager_forNominallySpacedGlyphRange_packedGlyphs_count_,  # noqa: B950
            2,
            b"n^v",
        )
        self.assertArgSizeInArg(
            AppKit.NSTypesetter.printingAdjustmentInLayoutManager_forNominallySpacedGlyphRange_packedGlyphs_count_,  # noqa: B950
            2,
            3,
        )



import AppKit
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSSliderCell(TestCase):
    def testConstants(self):
        self.assertEqual(AppKit.NSTickMarkBelow, 0)
        self.assertEqual(AppKit.NSTickMarkAbove, 1)
        self.assertEqual(AppKit.NSTickMarkLeft, AppKit.NSTickMarkAbove)
        self.assertEqual(AppKit.NSTickMarkRight, AppKit.NSTickMarkBelow)

        self.assertEqual(AppKit.NSLinearSlider, 0)
        self.assertEqual(AppKit.NSCircularSlider, 1)

        self.assertEqual(AppKit.NSTickMarkPositionBelow, 0)
        self.assertEqual(AppKit.NSTickMarkPositionAbove, 1)
        self.assertEqual(
            AppKit.NSTickMarkPositionLeading, AppKit.NSTickMarkPositionAbove
        )
        self.assertEqual(
            AppKit.NSTickMarkPositionTrailing, AppKit.NSTickMarkPositionBelow
        )

        self.assertEqual(AppKit.NSSliderTypeLinear, 0)
        self.assertEqual(AppKit.NSSliderTypeCircular, 1)

    def testMethods(self):
        self.assertResultIsBOOL(AppKit.NSSliderCell.prefersTrackingUntilMouseUp)
        self.assertArgIsBOOL(AppKit.NSSliderCell.knobRectFlipped_, 0)
        self.assertArgIsBOOL(AppKit.NSSliderCell.drawBarInside_flipped_, 1)
        self.assertResultIsBOOL(AppKit.NSSliderCell.allowsTickMarkValuesOnly)
        self.assertArgIsBOOL(AppKit.NSSliderCell.setAllowsTickMarkValuesOnly_, 0)

    @min_os_level("10.9")
    def testMethods10_9(self):
        self.assertArgIsBOOL(AppKit.NSSliderCell.barRectFlipped_, 0)



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSFileCoordinator(TestCase):
    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(
            Foundation.NSFileCoordinatorReadingImmediatelyAvailableMetadataOnly, 1 << 2
        )
        self.assertEqual(Foundation.NSFileCoordinatorReadingForUploading, 1 << 3)
        self.assertEqual(
            Foundation.NSFileCoordinatorWritingContentIndependentMetadataOnly, 1 << 4
        )

    @min_os_level("10.7")
    def testConstants(self):
        self.assertEqual(Foundation.NSFileCoordinatorReadingWithoutChanges, 1 << 0)
        self.assertEqual(
            Foundation.NSFileCoordinatorReadingResolvesSymbolicLink, 1 << 1
        )
        self.assertEqual(Foundation.NSFileCoordinatorWritingForDeleting, 1 << 0)
        self.assertEqual(Foundation.NSFileCoordinatorWritingForMoving, 1 << 1)
        self.assertEqual(Foundation.NSFileCoordinatorWritingForMerging, 1 << 2)
        self.assertEqual(Foundation.NSFileCoordinatorWritingForReplacing, 1 << 3)

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.coordinateAccessWithIntents_queue_byAccessor_,
            2,
            b"v@",
        )

    @min_os_level("10.7")
    def testMethods(self):
        self.assertArgIsOut(
            Foundation.NSFileCoordinator.coordinateReadingItemAtURL_options_error_byAccessor_,
            2,
        )
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.coordinateReadingItemAtURL_options_error_byAccessor_,
            3,
            b"v@",
        )

        self.assertArgIsOut(
            Foundation.NSFileCoordinator.coordinateWritingItemAtURL_options_error_byAccessor_,
            2,
        )
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.coordinateWritingItemAtURL_options_error_byAccessor_,
            3,
            b"v@",
        )

        self.assertArgIsOut(
            Foundation.NSFileCoordinator.coordinateReadingItemAtURL_options_writingItemAtURL_options_error_byAccessor_,  # noqa: B950
            4,
        )
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.coordinateReadingItemAtURL_options_writingItemAtURL_options_error_byAccessor_,  # noqa: B950
            5,
            b"v@@",
        )

        self.assertArgIsOut(
            Foundation.NSFileCoordinator.coordinateWritingItemAtURL_options_writingItemAtURL_options_error_byAccessor_,  # noqa: B950
            4,
        )
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.coordinateWritingItemAtURL_options_writingItemAtURL_options_error_byAccessor_,  # noqa: B950
            5,
            b"v@@",
        )

        self.assertArgIsOut(
            Foundation.NSFileCoordinator.prepareForReadingItemsAtURLs_options_writingItemsAtURLs_options_error_byAccessor_,  # noqa: B950
            4,
        )
        self.assertArgIsBlock(
            Foundation.NSFileCoordinator.prepareForReadingItemsAtURLs_options_writingItemsAtURLs_options_error_byAccessor_,  # noqa: B950
            5,
            b"v@?",
        )  # FIXME: Need test for the "nested" block



import Foundation
import objc
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestNSPredicate(TestCase):
    def testSimple(self):
        pred = Foundation.NSPredicate.predicateWithFormat_("a == 42")
        self.assertEqual(pred.predicateFormat(), "a == 42")

    def testFormat(self):
        pred = Foundation.NSPredicate.predicateWithFormat_("a == %d", 99)
        self.assertEqual(pred.predicateFormat(), "a == 99")

    def testBadFormat(self):
        self.assertRaises(
            ValueError, Foundation.NSPredicate.predicateWithFormat_, "a == %d"
        )

    def testMethods(self):
        self.assertArgIsBOOL(Foundation.NSPredicate.predicateWithValue_, 0)
        self.assertResultIsBOOL(Foundation.NSPredicate.evaluateWithObject_)
        self.assertResultIsBOOL(
            Foundation.NSPredicate.evaluateWithObject_substitutionVariables_
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsBlock(
            Foundation.NSPredicate.predicateWithBlock_, 0, objc._C_NSBOOL + b"@@"
        )



import Foundation
from PyObjCTools.TestSupport import TestCase, min_os_level, min_sdk_level
import objc


class TestNSFileManagerHelper(Foundation.NSObject):
    def fileManager_shouldProceedAfterError_(self, a, b):
        return 1

    def fileManager_willProcessPath_(self, a, b):
        pass

    def fileManager_shouldCopyItemAtPath_toPath_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_copyingItemAtPath_toPath_(self, a, b, c, d):
        return 1

    def fileManager_shouldMoveItemAtPath_toPath_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_movingItemAtPath_toPath_(self, a, b, c, d):
        return 1

    def fileManager_shouldLinkItemAtPath_toPath_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_linkingItemAtPath_toPath_(self, a, b, c, d):
        return 1

    def fileManager_shouldRemoveItemAtPath_(self, a, b):
        return 1

    def fileManager_shouldProceedAfterError_removingItemAtPath_(self, a, b, c):
        return 1

    def fileManager_shouldCopyItemAtURL_toURL_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_copyingItemAtURL_toURL_(self, a, b, c, d):
        return 1

    def fileManager_shouldMoveItemAtURL_toURL_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_movingItemAtURL_toURL_(self, a, b, c, d):
        return 1

    def fileManager_shouldLinkItemAtURL_toURL_(self, a, b, c):
        return 1

    def fileManager_shouldProceedAfterError_linkingItemAtURL_toURL_(self, a, b, c, d):
        return 1

    def fileManager_shouldRemoveItemAtURL_(self, a, b):
        return 1

    def fileManager_shouldProceedAfterError_removingItemAtURL_(self, a, b, c):
        return 1


class TestNSFileManager(TestCase):
    def testConstants(self):
        self.assertEqual(
            Foundation.NSFoundationVersionWithFileManagerResourceForkSupport, 412
        )

        self.assertIsInstance(Foundation.NSFileType, str)
        self.assertIsInstance(Foundation.NSFileTypeDirectory, str)
        self.assertIsInstance(Foundation.NSFileTypeRegular, str)
        self.assertIsInstance(Foundation.NSFileTypeSymbolicLink, str)
        self.assertIsInstance(Foundation.NSFileTypeSocket, str)
        self.assertIsInstance(Foundation.NSFileTypeCharacterSpecial, str)
        self.assertIsInstance(Foundation.NSFileTypeBlockSpecial, str)
        self.assertIsInstance(Foundation.NSFileTypeUnknown, str)
        self.assertIsInstance(Foundation.NSFileSize, str)
        self.assertIsInstance(Foundation.NSFileModificationDate, str)
        self.assertIsInstance(Foundation.NSFileReferenceCount, str)
        self.assertIsInstance(Foundation.NSFileDeviceIdentifier, str)
        self.assertIsInstance(Foundation.NSFileOwnerAccountName, str)
        self.assertIsInstance(Foundation.NSFileGroupOwnerAccountName, str)
        self.assertIsInstance(Foundation.NSFilePosixPermissions, str)
        self.assertIsInstance(Foundation.NSFileSystemNumber, str)
        self.assertIsInstance(Foundation.NSFileSystemFileNumber, str)
        self.assertIsInstance(Foundation.NSFileExtensionHidden, str)
        self.assertIsInstance(Foundation.NSFileHFSCreatorCode, str)
        self.assertIsInstance(Foundation.NSFileHFSTypeCode, str)
        self.assertIsInstance(Foundation.NSFileImmutable, str)
        self.assertIsInstance(Foundation.NSFileAppendOnly, str)
        self.assertIsInstance(Foundation.NSFileCreationDate, str)
        self.assertIsInstance(Foundation.NSFileOwnerAccountID, str)
        self.assertIsInstance(Foundation.NSFileGroupOwnerAccountID, str)
        self.assertIsInstance(Foundation.NSFileBusy, str)
        self.assertIsInstance(Foundation.NSFileSystemSize, str)
        self.assertIsInstance(Foundation.NSFileSystemFreeSize, str)
        self.assertIsInstance(Foundation.NSFileSystemNodes, str)
        self.assertIsInstance(Foundation.NSFileSystemFreeNodes, str)

    @min_os_level("10.13")
    def testConstantsMissingOn10_9(self):
        self.assertIsInstance(Foundation.NSFileProtectionKey, str)
        self.assertIsInstance(Foundation.NSFileProtectionNone, str)
        self.assertIsInstance(Foundation.NSFileProtectionComplete, str)
        self.assertIsInstance(Foundation.NSFileProtectionCompleteUnlessOpen, str)
        self.assertIsInstance(
            Foundation.NSFileProtectionCompleteUntilFirstUserAuthentication, str
        )

    @min_os_level("10.6")
    def testConstants10_6(self):
        self.assertEqual(Foundation.NSVolumeEnumerationSkipHiddenVolumes, 1 << 1)
        self.assertEqual(Foundation.NSVolumeEnumerationProduceFileReferenceURLs, 1 << 2)

        self.assertEqual(
            Foundation.NSDirectoryEnumerationSkipsSubdirectoryDescendants, 1 << 0
        )
        self.assertEqual(
            Foundation.NSDirectoryEnumerationSkipsPackageDescendants, 1 << 1
        )
        self.assertEqual(Foundation.NSDirectoryEnumerationSkipsHiddenFiles, 1 << 2)

        self.assertEqual(
            Foundation.NSFileManagerItemReplacementUsingNewMetadataOnly, 1 << 0
        )
        self.assertEqual(
            Foundation.NSFileManagerItemReplacementWithoutDeletingBackupItem, 1 << 1
        )

    @min_os_level("10.8")
    def testConstants10_8(self):
        self.assertIsInstance(Foundation.NSUbiquityIdentityDidChangeNotification, str)

    @min_os_level("10.10")
    def testConstants10_10(self):
        self.assertEqual(Foundation.NSURLRelationshipContains, 0)
        self.assertEqual(Foundation.NSURLRelationshipSame, 1)
        self.assertEqual(Foundation.NSURLRelationshipOther, 2)

    @min_os_level("10.11")
    def testConstants10_11(self):
        self.assertEqual(
            Foundation.NSFileManagerUnmountAllPartitionsAndEjectDisk, 1 << 0
        )
        self.assertEqual(Foundation.NSFileManagerUnmountWithoutUI, 1 << 1)

        self.assertIsInstance(
            Foundation.NSFileManagerUnmountDissentingProcessIdentifierErrorKey, str
        )

    @min_os_level("10.15")
    def testConstants10_15(self):
        self.assertEqual(
            Foundation.NSDirectoryEnumerationIncludesDirectoriesPostOrder, 1 << 3
        )
        self.assertEqual(
            Foundation.NSDirectoryEnumerationProducesRelativePathURLs, 1 << 4
        )

    @min_os_level("11.0")
    def testConstants11_0(self):
        self.assertIsInstance(Foundation.NSURLFileProtectionComplete, str)
        self.assertIsInstance(Foundation.NSURLFileProtectionCompleteUnlessOpen, str)
        self.assertIsInstance(
            Foundation.NSURLFileProtectionCompleteUntilFirstUserAuthentication, str
        )

    @min_os_level("10.6")
    def testMethods10_6(self):
        self.assertArgIsOut(
            Foundation.NSFileManager.contentsOfDirectoryAtURL_includingPropertiesForKeys_options_error_,  # noqa: B950
            3,
        )
        self.assertArgIsBOOL(
            Foundation.NSFileManager.URLForDirectory_inDomain_appropriateForURL_create_error_,
            3,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.URLForDirectory_inDomain_appropriateForURL_create_error_,
            4,
        )

        self.assertResultIsBOOL(Foundation.NSFileManager.copyItemAtURL_toURL_error_)
        self.assertArgIsOut(Foundation.NSFileManager.copyItemAtURL_toURL_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.moveItemAtURL_toURL_error_)
        self.assertArgIsOut(Foundation.NSFileManager.moveItemAtURL_toURL_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.linkItemAtURL_toURL_error_)
        self.assertArgIsOut(Foundation.NSFileManager.linkItemAtURL_toURL_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.removeItemAtURL_error_)
        self.assertArgIsOut(Foundation.NSFileManager.removeItemAtURL_error_, 1)

        self.assertArgIsBlock(
            Foundation.NSFileManager.enumeratorAtURL_includingPropertiesForKeys_options_errorHandler_,  # noqa: B950
            3,
            objc._C_NSBOOL + b"@@",
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.replaceItemAtURL_withItemAtURL_backupItemName_options_resultingItemURL_error_  # noqa: B950
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.replaceItemAtURL_withItemAtURL_backupItemName_options_resultingItemURL_error_,  # noqa: B950
            4,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.replaceItemAtURL_withItemAtURL_backupItemName_options_resultingItemURL_error_,  # noqa: B950
            5,
        )

    @min_os_level("10.7")
    def testMethods10_7(self):
        self.assertResultIsBOOL(
            Foundation.NSFileManager.createDirectoryAtURL_withIntermediateDirectories_attributes_error_  # noqa: B950
        )
        self.assertArgIsBOOL(
            Foundation.NSFileManager.createDirectoryAtURL_withIntermediateDirectories_attributes_error_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.createDirectoryAtURL_withIntermediateDirectories_attributes_error_,  # noqa: B950
            3,
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.createSymbolicLinkAtURL_withDestinationURL_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.createSymbolicLinkAtURL_withDestinationURL_error_,
            2,
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.setUbiquitous_itemAtURL_destinationURL_error_
        )
        self.assertArgIsBOOL(
            Foundation.NSFileManager.setUbiquitous_itemAtURL_destinationURL_error_, 0
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.setUbiquitous_itemAtURL_destinationURL_error_, 3
        )
        self.assertResultIsBOOL(Foundation.NSFileManager.isUbiquitousItemAtURL_)

        self.assertResultIsBOOL(
            Foundation.NSFileManager.startDownloadingUbiquitousItemAtURL_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.startDownloadingUbiquitousItemAtURL_error_, 1
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.evictUbiquitousItemAtURL_error_
        )
        self.assertArgIsOut(Foundation.NSFileManager.evictUbiquitousItemAtURL_error_, 1)

        self.assertArgIsOut(
            Foundation.NSFileManager.URLForPublishingUbiquitousItemAtURL_expirationDate_error_,
            1,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.URLForPublishingUbiquitousItemAtURL_expirationDate_error_,
            2,
        )

    @min_os_level("10.8")
    def testMethods10_8(self):
        self.assertResultIsBOOL(
            Foundation.NSFileManager.trashItemAtURL_resultingItemURL_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.trashItemAtURL_resultingItemURL_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.trashItemAtURL_resultingItemURL_error_, 2
        )

    @min_os_level("10.10")
    def testMethods10_10(self):
        self.assertResultIsBOOL(
            Foundation.NSFileManager.getRelationship_ofDirectoryAtURL_toItemAtURL_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.getRelationship_ofDirectoryAtURL_toItemAtURL_error_,
            0,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.getRelationship_ofDirectoryAtURL_toItemAtURL_error_,
            3,
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.getRelationship_ofDirectory_inDomain_toItemAtURL_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.getRelationship_ofDirectory_inDomain_toItemAtURL_error_,
            0,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.getRelationship_ofDirectory_inDomain_toItemAtURL_error_,
            4,
        )

    def testOutput(self):
        obj = Foundation.NSFileManager.defaultManager()
        m = obj.setAttributes_ofItemAtPath_error_.__metadata__()
        self.assertTrue(m["arguments"][4]["type"].startswith(b"o^"))

        m = (
            obj.createDirectoryAtPath_withIntermediateDirectories_attributes_error_.__metadata__()  # noqa: B950
        )
        self.assertEqual(m["arguments"][3]["type"], b"Z")
        self.assertTrue(m["arguments"][5]["type"].startswith(b"o^"))

        m = obj.contentsOfDirectoryAtPath_error_.__metadata__()
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        m = obj.subpathsOfDirectoryAtPath_error_.__metadata__()
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        m = obj.attributesOfItemAtPath_error_.__metadata__()
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        m = obj.attributesOfFileSystemForPath_error_.__metadata__()
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        m = obj.createSymbolicLinkAtPath_withDestinationPath_error_.__metadata__()
        self.assertTrue(m["arguments"][4]["type"].startswith(b"o^"))

        m = obj.destinationOfSymbolicLinkAtPath_error_.__metadata__()
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

        m = obj.copyItemAtPath_toPath_error_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        self.assertTrue(m["arguments"][4]["type"].startswith(b"o^"))

        m = obj.moveItemAtPath_toPath_error_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        self.assertTrue(m["arguments"][4]["type"].startswith(b"o^"))

        m = obj.linkItemAtPath_toPath_error_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        self.assertTrue(m["arguments"][4]["type"].startswith(b"o^"))

        m = obj.removeItemAtPath_error_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        self.assertTrue(m["arguments"][3]["type"].startswith(b"o^"))

    def testProtocols(self):
        class FileManagerTest1(Foundation.NSObject):
            def fileManager_shouldCopyItemAtPath_toPath_(self, fm, src, dst):
                return True

            def fileManager_shouldProceedAfterError_copyingItemAtPath_toPath_(
                self, fm, error, src, dst
            ):
                return True

            def fileManager_shouldMoveItemAtPath_toPath_(self, fm, src, dst):
                return True

            def fileManager_shouldProceedAfterError_movingItemAtPath_toPath_(
                self, fm, error, src, dst
            ):
                return True

            def fileManager_shouldLinkItemAtPath_toPath_(self, fm, src, dst):
                return True

            def fileManager_shouldProceedAfterError_linkingItemAtPath_toPath_(
                self, fm, error, src, dst
            ):
                return True

            def fileManager_shouldRemoveItemAtPath_(self, fm, src):
                return True

            def fileManager_shouldProceedAfterError_removingItemAtPath_(
                self, fm, error, src
            ):
                return True

        obj = FileManagerTest1.alloc().init()
        m = obj.fileManager_shouldCopyItemAtPath_toPath_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        m = (
            obj.fileManager_shouldProceedAfterError_copyingItemAtPath_toPath_.__metadata__()
        )
        self.assertEqual(m["retval"]["type"], b"Z")
        m = obj.fileManager_shouldMoveItemAtPath_toPath_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        m = (
            obj.fileManager_shouldProceedAfterError_movingItemAtPath_toPath_.__metadata__()
        )
        self.assertEqual(m["retval"]["type"], b"Z")
        m = obj.fileManager_shouldLinkItemAtPath_toPath_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        m = (
            obj.fileManager_shouldProceedAfterError_linkingItemAtPath_toPath_.__metadata__()
        )
        self.assertEqual(m["retval"]["type"], b"Z")
        m = obj.fileManager_shouldRemoveItemAtPath_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")
        m = obj.fileManager_shouldProceedAfterError_removingItemAtPath_.__metadata__()
        self.assertEqual(m["retval"]["type"], b"Z")

    @min_os_level("10.5")
    def testMethods10_5(self):
        self.assertResultIsBOOL(
            Foundation.NSFileManager.setAttributes_ofItemAtPath_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.setAttributes_ofItemAtPath_error_, 2
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.createDirectoryAtPath_withIntermediateDirectories_attributes_error_  # noqa: B950
        )
        self.assertArgIsBOOL(
            Foundation.NSFileManager.createDirectoryAtPath_withIntermediateDirectories_attributes_error_,  # noqa: B950
            1,
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.createDirectoryAtPath_withIntermediateDirectories_attributes_error_,  # noqa: B950
            3,
        )

        self.assertArgIsOut(
            Foundation.NSFileManager.contentsOfDirectoryAtPath_error_, 1
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.subpathsOfDirectoryAtPath_error_, 1
        )
        self.assertArgIsOut(Foundation.NSFileManager.attributesOfItemAtPath_error_, 1)
        self.assertArgIsOut(
            Foundation.NSFileManager.attributesOfFileSystemForPath_error_, 1
        )

        self.assertResultIsBOOL(
            Foundation.NSFileManager.createSymbolicLinkAtPath_withDestinationPath_error_
        )
        self.assertArgIsOut(
            Foundation.NSFileManager.createSymbolicLinkAtPath_withDestinationPath_error_,
            2,
        )

        self.assertArgIsOut(
            Foundation.NSFileManager.destinationOfSymbolicLinkAtPath_error_, 1
        )

        self.assertResultIsBOOL(Foundation.NSFileManager.copyItemAtPath_toPath_error_)
        self.assertArgIsOut(Foundation.NSFileManager.copyItemAtPath_toPath_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.moveItemAtPath_toPath_error_)
        self.assertArgIsOut(Foundation.NSFileManager.moveItemAtPath_toPath_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.linkItemAtPath_toPath_error_)
        self.assertArgIsOut(Foundation.NSFileManager.linkItemAtPath_toPath_error_, 2)
        self.assertResultIsBOOL(Foundation.NSFileManager.removeItemAtPath_error_)
        self.assertArgIsOut(Foundation.NSFileManager.removeItemAtPath_error_, 1)

    def testMethods(self):
        self.assertArgIsBOOL(
            Foundation.NSFileManager.fileAttributesAtPath_traverseLink_, 1
        )
        self.assertResultIsBOOL(Foundation.NSFileManager.changeFileAttributes_atPath_)
        self.assertResultIsBOOL(
            Foundation.NSFileManager.createSymbolicLinkAtPath_pathContent_
        )
        self.assertResultIsBOOL(
            Foundation.NSFileManager.createDirectoryAtPath_attributes_
        )
        self.assertResultIsBOOL(Foundation.NSFileManager.linkPath_toPath_handler_)
        self.assertResultIsBOOL(Foundation.NSFileManager.copyPath_toPath_handler_)
        self.assertResultIsBOOL(Foundation.NSFileManager.movePath_toPath_handler_)
        self.assertResultIsBOOL(Foundation.NSFileManager.removeFileAtPath_handler_)
        self.assertResultIsBOOL(Foundation.NSFileManager.changeCurrentDirectoryPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.fileExistsAtPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.fileExistsAtPath_isDirectory_)
        self.assertArgHasType(
            Foundation.NSFileManager.fileExistsAtPath_isDirectory_,
            1,
            b"o^" + objc._C_NSBOOL,
        )
        self.assertResultIsBOOL(Foundation.NSFileManager.isReadableFileAtPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.isWritableFileAtPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.isExecutableFileAtPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.isDeletableFileAtPath_)
        self.assertResultIsBOOL(Foundation.NSFileManager.contentsEqualAtPath_andPath_)
        self.assertResultIsBOOL(
            Foundation.NSFileManager.createFileAtPath_contents_attributes_
        )
        self.assertResultHasType(
            Foundation.NSFileManager.fileSystemRepresentationWithPath_,
            b"^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertResultIsNullTerminated(
            Foundation.NSFileManager.fileSystemRepresentationWithPath_
        )
        self.assertArgHasType(
            Foundation.NSFileManager.stringWithFileSystemRepresentation_length_,
            0,
            b"n^" + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgSizeInArg(
            Foundation.NSFileManager.stringWithFileSystemRepresentation_length_, 0, 1
        )

        self.assertResultIsBOOL(Foundation.NSDictionary.fileIsImmutable)
        self.assertResultIsBOOL(Foundation.NSDictionary.fileIsAppendOnly)
        self.assertResultIsBOOL(Foundation.NSDictionary.fileExtensionHidden)

    @min_os_level("10.11")
    def testMethods10_11(self):
        self.assertArgIsBlock(
            Foundation.NSFileManager.unmountVolumeAtURL_options_completionHandler_,
            2,
            b"v@",
        )

    @min_os_level("10.13")
    def testMethods10_13(self):
        self.assertArgIsBlock(
            Foundation.NSFileManager.getFileProviderMessageInterfacesForItemAtURL_completionHandler_,  # noqa: B950
            1,
            b"v@@",
        )

    @min_os_level("10.15")
    def testMethods10_15(self):
        self.assertResultIsBOOL(
            Foundation.NSDirectoryEnumerator.isEnumeratingDirectoryPostOrder
        )

    def testProtocolsMethods(self):
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldCopyItemAtPath_toPath_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_copyingItemAtPath_toPath_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldMoveItemAtPath_toPath_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_movingItemAtPath_toPath_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldLinkItemAtPath_toPath_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_linkingItemAtPath_toPath_  # noqa: B950
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldRemoveItemAtPath_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_removingItemAtPath_
        )

    @min_os_level("10.6")
    def testProtocols10_6(self):

        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldCopyItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_copyingItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldMoveItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_movingItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldLinkItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_linkingItemAtURL_toURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldRemoveItemAtURL_
        )
        self.assertResultIsBOOL(
            TestNSFileManagerHelper.fileManager_shouldProceedAfterError_removingItemAtURL_
        )

    @min_sdk_level("10.10")
    def testProtocols10_10(self):
        objc.protocolNamed("NSFileManagerDelegate")
